<?php
class Mappress_Filter extends Mappress_Obj {
	public
		$key,
		$exclude = array(),
		$format,
		$include = array(),
		$label,
		$meta_key,
		$meta_type,
		$placeholder,
		$sort,
		$values = array()
	;

	function __construct($atts) {
		$this->update($atts);
		$this->format = ($this->format) ? $this->format : 'checkboxes';
		$this->label = ($this->label) ? $this->label : $this->get_label();
		$this->values = $this->get_values();
	}

	static function register() {
		add_action('wp_ajax_mapp_filters_get', array(__CLASS__, 'ajax_filters_get'));
		add_action('wp_ajax_nopriv_mapp_filters_get', array(__CLASS__, 'ajax_filters_get'));
	}

	static function ajax_filters_get() {
		check_ajax_referer('mappress', 'nonce');
		$type = (isset($_GET['type'])) ? $_GET['type'] : 'post';
		$name = (isset($_GET['name'])) ? $_GET['name'] : null;

		// Only one type of filter is returned
		$filters = Mappress::$options->filters[$type];
		$results = array();
		foreach($filters as $atts)
			$results[] = new Mappress_Filter($atts);
			
		// Allow users to modify filters (for example, by map name)
		foreach($results as $filter)
			$filter->values = apply_filters('mappress_filter_values', $filter->values, $name);
		
		Mappress::ajax_response('OK', $results);
	}

	function get_icon($value) {
		$autoicons = (object) Mappress::$options->autoicons;
		return ($autoicons && isset($autoicons->key) && $autoicons->key == $this->key && isset($autoicons->values[$value])) ? $autoicons->values[$value] : null;
	}

	function get_label() {
		if (!$this->key)
			return '???';

		switch($this->key) {
			case 'meta' :
				$label = $this->key;
				break;

			case 'post' :
				$label = __('Post type', 'mappress-google-maps-for-wordpress');
				break;

			case 'role' :
				$label = __('Role', 'mappress-google-maps-for-wordpress');
				break;

			case 'text' :
				$label = __('Text search', 'mappress-google-maps-for-wordpress');
				break;

			default :
				$taxonomy = get_taxonomy($this->key);
				if ($taxonomy)
					return ($this->format == 'select') ? $taxonomy->labels->singular_name : $taxonomy->label;
				else
					return '???';
				break;
		}
		return ucfirst($label);
	}

	function get_taxonomy_terms() {
		$args = array(
			'count' => false,
			'exclude' => 1,
			'hide_empty' => true,
			'indent' => true,
			'orderby' => ($this->sort == 'slug') ? 'slug' : 'name'
		);
		$terms = get_terms($this->key, $args);
		$results = array();

		if (is_array($terms)) {
			$walker = new Mappress_Walker($args);
			$walk = $walker->walk($terms, 0, $args);
			if (is_array($walk))
				$results = $walk;
		}
		return $results;
	}

	static function get_url_params() {
		global $wp_query;
		$filter_types = (Mappress::$options->filters) ? Mappress::$options->filters : array();
		
		// Get keys for all filter type (post, user, poi)
		$params = array();

		foreach($filter_types as $filter_type => $filters) {
			foreach($filters as $filter)  {
				$key = (!isset($filter['key'])) ? $filter['key'] : null;
				if (!$key)
					continue;					
				$value = (isset($_REQUEST[$key])) ? $_REQUEST[$key] : '';
				if (!$value) {
					// If variable wasn't in request, see if it's in query vars
					$tax = get_taxonomy($key);
					if ($tax)
						$value = get_query_var($tax->query_var);
					else if ($key == 'post')
						$value = get_query_var('post_type');
					else if ($key == 'text')
						$value = get_query_var('s');
				}
				if ($value) {
					if (strpos($value, ',') >= 0) {
						$value = explode(',', $value);
						if ($filter['format'] != 'checkboxes')
							$value = $value[0];
					}
					$params[$key] = $value;
				}
			}
		}
		$params = apply_filters('mappress_url_params', $params);
		return $params;
	}
		
	function get_values() {
		global $wpdb;

		$values = array();

		switch($this->key) {
			case 'meta' :
				$meta_values = $wpdb->get_col($wpdb->prepare("SELECT DISTINCT meta_value FROM $wpdb->postmeta WHERE meta_key = %s AND meta_value != '' ORDER BY meta_value", $this->key));
				$values = ($meta_values) ? array_combine($meta_values, $meta_values) : array();
				break;

			case 'post' :
				$types = get_post_types(array('public' => true), 'objects');
				foreach($types as $key => $type)
					$values[$key] = $type->label;
				break;

			case 'role' :
				global $wp_roles;
				$roles = $wp_roles->roles;
				foreach($roles as $key => $role)
					$values[$key] = $role['name'];
				break;

			case 'text' :
				$values = null;
				break;

			default :
				$values = $this->get_taxonomy_terms();
				break;
		}

		// Filter by include/exclude and add icons
		if ($values) {
			$results = array();
			foreach($values as $value => $label) {
				if ( (empty($this->include) || in_array($value, $this->include)) && (empty($this->exclude) || !in_array($value, $this->exclude)) )
					$results[] = array('value' => $value, 'iconid' => $this->get_icon($value), 'label' => $label);
			}
		} else {
			$results = null;
		}
		
		return $results;
	}
}

/**
* Walker for taxonomy values.
* Call with array of terms objects: $walker->walk($terms, 0);
* Returns array of (term slug => name)
*/
class Mappress_Walker extends Walker {
	public $db_fields = array ('parent' => 'parent', 'id' => 'term_id');

	function start_el( &$output, $term, $depth = 0, $args = array(), $id = 0 ) {
		if (!is_array($output))
			$output = array();

		// If 'indent' set, use spaces (for hierarchical lists like taxonomies)
		$indent = (isset($args['indent']) && $args['indent']) ? str_repeat('&mdash;', $depth) . ' ' : '';
		$output[$term->slug] = $indent . $term->name;
	}
}
?>