<?php
class Mappress_Frontend {

	static function register() {
		if (class_exists('acf')) {
			// Add acf setting to allow specifying acf field name for mashups 
			add_filter('mappress_options', array(__CLASS__, 'acf_mappress_options'));
			
			// Process array meta values for ACF map custom fields
			add_filter('mappress_meta_value', array(__CLASS__, 'acf_mappress_meta_value'), 10, 2);

			// If mashup field name specified, include all acf fields in all mashups
			if (Mappress::$options->acf) 
				add_filter('mappress_query_pois', array(__CLASS__, 'acf_mappress_query_pois'), 10, 2);
		}
	}

	static function acf_mappress_meta_value($poi, $meta_value) {
		// ACF saves POIs as a serialized array, with htmlencoded address
		// No viewport is provided.  Geocoding is expensive, so for now it's assumed that POIs are street addresses.
		if (!is_array($meta_value))
			return null;

		$meta_value = (object) $meta_value;
		if (!$meta_value->lat || !$meta_value->lng || !$meta_value->address)
			return new WP_Error('address', __('Address or lat/lng was missing in ACF field', 'mappress-google-maps-for-wordpress'));
		$poi->point['lat'] = $meta_value->lat;
		$poi->point['lng'] = $meta_value->lng;
		$poi->address = html_entity_decode($meta_value->address);

		// Because lat,lng already exist, geocoding just parses the title & body
		$poi->geocode();
		return $poi;
	}

	static function acf_mappress_options($l10n) {
		$l10n['options']['acf'] = Mappress::$options->acf;
		$l10n['options']['acfActive'] = true;
		return $l10n;
	}

	static function acf_mappress_query_pois($pois, $query) {
		global $wpdb;
		
		// Set up meta query for acf field        
		$query['meta_query'] = array(
			'relation' => 'AND',
			array('key' => Mappress::$options->acf, 'compare' => 'exists')
		);

		// If there was an existing meta_key in the query, add it to the new meta_query
		$old_key = (isset($query['meta_key'])) ? $query['meta_key'] : null;
		$old_value = (isset($query['meta_value'])) ? $query['meta_value'] : null;
		if ($old_key && $old_value) {
			$query['meta_query'][] = array('key' => $old_key, 'value' => $old_value);
			unset($query['meta_key']);
		}
		
		// Re-run query, without filters that join to maps table
		//$query['meta_key'] = Mappress::$options->acf;
		$wpq = new WP_Query($query);

		// Gather POIs
		foreach($wpq->posts as $postid) {
			// ACF stores data as serialized array with 1 entry
			$meta_value = get_post_meta($postid, Mappress::$options->acf, true);
			if (empty($meta_value) || !is_array($meta_value))
				continue;

			$meta_value = (object) $meta_value;
			if (!isset($meta_value->lat) || !isset($meta_value->lng) || !isset($meta_value->address))
				continue;

			$poi = new Mappress_Poi();
			$poi->point['lat'] = $meta_value->lat;
			$poi->point['lng'] = $meta_value->lng;
			$poi->address = html_entity_decode($meta_value->address);
			$poi->geocode();		// Just sets poi title & body

			// Use post object for WP get_permalink()
			$post = get_post($postid);
			$post->filter = 'raw';
			Mappress_Query::merge_postdata($poi, $post);
			$pois[] = $poi;
		}
		return $pois;
	}
}
?>