<?php
class Mappress_Icons {
	static  $fonts = array(),
			$icons_dir,
			$icons_url,
			$standard_icons_url,
			$user_icons = array(),
			$shapes_dir,
			$shapes_url,
			$standard_icons = array ('blue-dot', 'ltblue-dot', 'green-dot', 'pink-dot', 'purple-dot', 'red-dot', 'yellow-dot', 'blue', 'green', 'lightblue', 'pink', 'purple', 'red', 'yellow', 'blue-pushpin', 'grn-pushpin', 'ltblu-pushpin', 'pink-pushpin', 'purple-pushpin', 'red-pushpin', 'ylw-pushpin', 'bar', 'coffeehouse', 'man', 'wheel_chair_accessible', 'woman', 'restaurant', 'snack_bar', 'parkinglot', 'bus', 'cabs', 'ferry', 'helicopter', 'plane', 'rail', 'subway', 'tram', 'truck', 'info', 'info_circle', 'rainy', 'sailing', 'ski', 'snowflake_simple', 'swimming', 'water', 'fishing', 'flag', 'marina', 'campfire', 'campground', 'cycling', 'golfer', 'hiker', 'horsebackriding', 'motorcycling', 'picnic', 'POI', 'rangerstation', 'sportvenue', 'toilets', 'trail', 'tree', 'arts', 'conveniencestore', 'dollar', 'electronics', 'euro', 'gas', 'grocerystore', 'homegardenbusiness', 'mechanic', 'movies', 'realestate', 'salon', 'shopping', 'yen', 'caution', 'earthquake', 'fallingrocks', 'firedept', 'hospitals', 'lodging', 'phone', 'partly_cloudy', 'police', 'postoffice-us', 'sunny', 'volcano', 'camera', 'webcam', 'iimm1-blue', 'iimm1-green', 'iimm1-orange', 'iimm1-red', 'iimm2-blue', 'iimm2-green', 'iimm2-orange', 'iimm2-red');

	static function register() {
		// Create directories
		$upload = wp_upload_dir();
		$basedir = $upload['basedir'] . "/mappress";
		$baseurl = $upload['baseurl'] . "/mappress";
		wp_mkdir_p($basedir);
		wp_mkdir_p($basedir . "/icons");
		self::$icons_dir = $basedir . "/icons/";
		self::$icons_url = $baseurl . "/icons/";
		self::$standard_icons_url = Mappress::$baseurl . '/pro/standard_icons/';
		self::$shapes_dir = Mappress::$basedir . '/pro/shapes/';
		self::$shapes_url = Mappress::$baseurl . '/pro/shapes/';

		self::$user_icons = self::get_user_icons();
		add_action('wp_ajax_mapp_icon_save', array(__CLASS__, 'ajax_save'));
		add_action('wp_ajax_mapp_icon_delete', array(__CLASS__, 'ajax_delete'));

		// Font CSS files to be queued
		$fonts = array(
			'map-icons' => array('version' => '1.0.0', 'url' => '{baseurl}/fonts/map-icons.css'),
		);

		// Some fonts need an array of URLs
		$font = (object) $fonts['map-icons'];
		$urls = (is_array($font->url)) ? $font->url : array($font->url);
		foreach($urls as $i => $url) {
			$url = str_replace('{baseurl}', Mappress::$baseurl, $url);
			wp_register_style("mappress-font-" . ($i + 1), $url, null, $font->version);
		}
	}

	static function ajax_delete() {
		check_ajax_referer('mappress', 'nonce');

		if (!current_user_can('manage_options'))
			Mappress::ajax_response('Not authorized');

		$args = json_decode(wp_unslash($_POST['data']));
		$iconids = $args->iconids;

		if (!$iconids)
			Mappress::ajax_response('Nothing selected');

		if (!is_array($iconids))
			$iconids = array($iconids);

		foreach($iconids as $iconid) {
			wp_delete_file(self::$icons_dir . $iconid);
			$i = array_search($iconid, self::$user_icons);
			if ($i !== false)
				array_splice(self::$user_icons, $i, 1);
		}

		Mappress::ajax_response('OK', self::get_user_icons());
	}

	static function ajax_save() {
		check_ajax_referer('mappress', 'nonce');

		if (!current_user_can('manage_options'))
			Mappress::ajax_response('Not authorized');

		$files = (isset($_FILES)) ? $_FILES : null;

		// Redirect uploads to MapPress directory
		add_filter( 'upload_dir', array(__CLASS__, 'upload_dir'));
		$filename = self::save_files($files);      

		if ($files)
			Mappress::ajax_response('OK', array('icons' => self::get_user_icons(), 'filename' => $filename));
		else
			Mappress::ajax_response('Missing image data');
	}

	static function copy_old_icons() {
		WP_Filesystem();
		$result = unzip_file(Mappress::$basedir . '/pro/old_icons/old_icons.zip', self::$icons_dir);
		if (is_wp_error($result)) {
			echo 'Error copying icons!  Please copy manually';
		}
	}

	/**
	* Get an icon image URL for output in the filter dropdown
	*/
	static function get($icon) {
		if (Mappress::$pro)
			$icon = (empty($icon)) ? Mappress::$options->defaultIcon : $icon;
		else
			$icon = null;
		$icon = (empty($icon)) ? 'red-dot' : $icon;
		$url = ( stristr($icon, '.svg') || stristr($icon, '.jpg') || stristr($icon, '.png') || stristr($icon, '.gif')) ? self::$icons_url . $icon : self::$standard_icons_url . $icon . '.png';
		return $url;
	}

	static function get_user_icons() {
		self::$user_icons = array();
		$files = @scandir(self::$icons_dir);
		if ($files) {
			// Filter all but images
			foreach($files as $file) {
				if ( !stristr($file, '.shadow') && ( stristr($file, '.svg') || stristr($file, '.png') || stristr($file, '.gif') || stristr($file, '.jpg')) )
					self::$user_icons[] = $file;
			}

			// Sort image names, ignoring suffixes
			usort(self::$user_icons, function($a, $b) {
				$no_suffix_a = str_ireplace(array('.svg', '.gif', '.png', 'jpg'), '', $a);
				$no_suffix_b = str_ireplace(array('.svg', '.gif', '.png', 'jpg'), '', $b);
				return strnatcmp($no_suffix_a, $no_suffix_b);
			});
		}
		return self::$user_icons;
	}

	static function save_files($files) {
		$last_filename = null;

		foreach($files as $file) {
			if ($file['error']) {
				$errnum = $file['error'];
				$msgs = array(
					'No error',
					'The uploaded file exceeds the upload_max_filesize directive in php.ini',
					'The uploaded file exceeds the MAX_FILE_SIZE directive that was specified in the HTML form.',
					'The uploaded file was only partially uploaded.',
					'No file was uploaded.',
					'Missing a temporary folder.',
					'Failed to write file to disk.',
					'A PHP extension stopped the file upload.'
				);
				$msg = (isset($msgs[$errnum])) ? $msgs[$errnum] : 'Unknown error';
				Mappress::ajax_response(__('Files error: ', 'mappress-google-maps-for-wordpress') . $msg);
			}

			// Redirect uploads to MapPress directory
			add_filter( 'upload_dir', array(__CLASS__, 'upload_dir'));
			$result = wp_upload_bits(wp_basename($file['name']), null, file_get_contents($file['tmp_name']));

			if ($result['error'])
				Mappress::ajax_response('Error in wp_upload_bits: ' . $result['error']);
			else
				$last_filename = basename($result['file']);
		}
		return $last_filename;
	}

	static function upload_dir($dir) {
		$dir['path'] = self::$icons_dir;
		$dir['url'] = '';
		$dir['subdir'] = '/mappress/icons';
		return $dir;
	}
}
?>