<?php
class Mappress_Import {

	function __construct($atts) {}

	static function register() {}

	// Get allowed columns (union of constants and any poiFields from settings)
	static function get_columns() {
		$standard = array('rowid', 'mapid', 'lat', 'lng', 'address', 'title', 'body', 'iconid');

		$custom = array();
		if (Mappress::$options->poiFields) {
			foreach(Mappress::$options->poiFields as $field)
				$custom[] = $field['key'];
		}

		return (object) array('standard' => $standard, 'custom' => $custom, 'all' => array_merge($standard, $custom));
	}

	static function import($request) {
		ob_start();
		$url = $request->get_param('url');
		$files = $request->get_file_params();
		$overwrite = $request->get_param('overwrite');

		// Convert overwrite boolean, pick out first (and only) file
		$overwrite = ($overwrite == 'true') ? true : false;
		$file = (isset($files['file'])) ? $files['file'] : null;

		if ($url && !ini_get("allow_url_fopen"))
			return Mappress::$api->rest_response(new WP_Error('import', __('Setting "allow_url_fopen=1" must in your php.ini file to process remote files.  Not all hosting services allow this.', 'mappress-google-maps-for-wordpress')));

		$filename = ($file) ? $file['tmp_name'] : $url;

		if (empty($filename))
			return Mappress::$api->rest_response(new WP_Error('import', 'Missing file name and URL'));

		$handle = @fopen($filename, "r");

		if ($handle == false)
			return Mappress::$api->rest_response(new WP_Error('import', __('Unable to open file', 'mappress-google-maps-for-wordpress')));

		$headers = fgetcsv($handle, 10000, ',');
		if ($headers == false)
			return Mappress::$api->rest_response(new WP_Error('import', __('Unable to read file headers', 'mappress-google-maps-for-wordpress')));

		// If file has a UTF BOM, strip it out
		if (substr($headers[0], 0, 3) == "\xef\xbb\xbf")
			$headers[0] = substr($headers[0], 3);

		// Always use lower case headers
		foreach($headers as &$header)
			$header = strtolower($header);

		$cols  = self::get_columns();

		// Read data[]
		$data = array();
		$i = 0;
		while (($line = fgetcsv($handle, 10000, ',')) !== false) {
			$i++;

			// Header count must match line column count
			if (count($headers) != count($line))
				return Mappress::$api->rest_response(new WP_Error('import', sprintf(__('Line %d column count doesn\'t match headers column count', 'mappress-google-maps-for-wordpress'), $i)));

			$line = array_combine($headers, $line);
			$row = (object) array_intersect_key($line, array_combine($cols->all, $cols->all));
			$row->rowid = $i;
			$data[] = $row;
		}

		usort($data, array(__CLASS__, 'sort_by_mapid'));

		/**
		* Output format: array of [data, logs] for each mapid
		* Logs are either by POI id or for null id (header)
		*/
		$results = array();
		while ($data) {
			$map = new Mappress_Map($data);

			// Overwrite only if mapid already exists
			$mapid = (isset($data[0]->mapid) && $data[0]->mapid) ? $data[0]->mapid : null;
			if ($overwrite && $mapid) {
				$exists = Mappress_Map::get($mapid);
				if ($exists)
					$map->mapid = $mapid;
			}

			// Postid = 0 means unattached.  Column 'title' is in both map & poi, so set map title to fixed string.
			$map->title = __('Import', 'mappress-google-maps-for-wordpress');
			$map->otype = 'post';
			$map->oid = 0;

			// All pois are returned, but map is only saved if they're all valid
			$raw_pois = self::process_pois($data, $results);
			$map->pois = array_filter($raw_pois, function ($poi) { return empty($poi->log); });
			$error_count = count($raw_pois) - count($map->pois);

			$result = $map->save();
			if ($result == false)
				$log = 'Internal error, Map could not be saved!';
			else
				$log = (isset($exists) && $exists) ? sprintf(__('Map %d updated.', 'mappress-google-maps-for-wordpress'), $map->mapid) : sprintf(__('Map %d saved.', 'mappress-google-maps-for-wordpress'), $map->mapid);

			$log .= ($error_count) ? ' ' . __('See data view for errors.', 'mappress-google-maps-for-wordpress') : '';

			// Return 'raw' pois, including pois with errors
			$map->pois = $raw_pois;
			$results[] = array('mapdata' => $map, 'log' => $log);
		}

		fclose($handle);
		return Mappress::$api->rest_response($results);
	}

	static function process_pois(&$data) {
		$pois = array();
		while ($data) {
			$row = array_shift($data);	// Pop
			$mapid = (isset($row->mapid) && $row->mapid) ? $row->mapid : null;

			if (isset($last_mapid) && $mapid != $last_mapid)
				return $pois;

			$last_mapid = (isset($row->mapid) && $row->mapid) ? $row->mapid : null;

			$poi = new Mappress_Poi($row);

			// Set poi data - note that any characters are allowed in column names
			$cols = self::get_columns();

			// For custom fields, split values with commas into arrays
			foreach((array) $cols->custom as $key) {
				if (isset($row->$key))
					$poi->data[$key] = explode(',', $row->$key);
			}

			// Lat/lng
			$lat = ($row->lat) ? (float) $row->lat : null;
			$lng = ($row->lng) ? (float) $row->lng : null;
			if (isset($lat) && isset($lng) && $lng > -180 && $lng < 180 && $lat > -90 && $lat < 90)
				$poi->point = array('lat' => $lat, 'lng' => $lng);
			else
				$poi->point = null;		// POI initializes point to (0,0) set reset it

			if ($poi->address || $poi->point) {
				$result = $poi->geocode();
				if (is_wp_error($result))
					$poi->log = __('Geocoding failed', 'mappress-google-maps-for-wordpress') . ' : ' . $result->get_error_message();
			} else {
				$poi->log = __('No address or lat/lng provided!', 'mappress-google-maps-for-wordpress');
			}
			$pois[] = $poi;
		}
		return $pois;
	}

	static function sort_by_mapid($a, $b) {
		// Sort by mapid, rowid
		$a_mapid = (isset($a->mapid)) ? $a->mapid : null;
		$b_mapid = (isset($b->mapid)) ? $b->mapid : null;
		if ($a_mapid == $b_mapid) {
			if ($a->rowid == $b->rowid)
				return 0;
			else
				return ($a->rowid > $b->rowid) ? 1 : -1;
		} else {
			return ($a_mapid > $b_mapid) ? 1 : -1;
		}
	}

	static function import_page() {
		?>
		<div class='mapp-import-page'>
			<div class='mapp-import-header'>
				<h1><?php _e('Import Maps', 'mappress-google-maps-for-wordpress'); ?></h1>
				<a target='_blank' href='https://mappresspro.com/wordpress/wp-content/uploads/example.csv'><?php _e('Sample file', 'mappress-google-maps-for-wordpress');?></a>
				<a target='_blank' href='https://mappresspro.com/mappress-documentation/#toc-importing-maps'><?php _e('Import documentation', 'mappress-google-maps-for-wordpress');?></a>
			</div>
			<div id="mapp-import-container"></div>
		</div>
		<?php
	}
}
?>