<?php
class Mappress_Widget_Map extends WP_Widget {

	var $defaults = array(
		'height' => 300,
		'mapid' => null,
		'other' => '',
		'poiList' => false,
		'widget_title' => 'MapPress Map',
		'width' => '100%'
	);

	function __construct() {
		parent::__construct(false, $name = 'MapPress Map');
	}

	static function register() {
		add_action('widgets_init', array(__CLASS__, 'widgets_init'));
	}

	static function widgets_init() {
		return register_widget("Mappress_Widget_Map");
	}

	function widget($args, $instance) {
		// Get widget attributes
		$atts = wp_parse_args($instance, $this->defaults);

		// Merge in any misc (shortcode) atts
		$other = shortcode_parse_atts($atts['other']);
		$other = Mappress::scrub_atts($other);
		$atts = array_merge($atts, $other);

		// Get the map html
		$html = Mappress::shortcode_map($atts);

		// If html is empty, show nothing
		if (empty($html))
			return;

		$title = (isset($instance['widget_title'])) ? $instance['widget_title'] : '';

		echo $args['before_widget'];
		echo $args['before_title'] . $title . $args['after_title'];
		echo $html;
		echo $args['after_widget'];
	}

	function update($new_instance, $old_instance) {
		$new_instance['poiList'] = (isset($new_instance['poiList'])) ? true : false;
		$new_instance['mapid'] = (isset($new_instance['mapid'])) ? (int) ($new_instance['mapid']) : null;
		return $new_instance;
	}

	function form($instance) {
		$args = (object) wp_parse_args($instance, $this->defaults);
		?>
			<p>
				<?php _e('Widget title', 'mappress-google-maps-for-wordpress'); ?>:
				<input class="widefat" id="<?php echo $this->get_field_id('widget_title'); ?>" name="<?php echo $this->get_field_name('widget_title'); ?>" type="text" value="<?php echo $args->widget_title ?>" />
			</p>

			<p>
				<?php _e('Map ID', 'mappress-google-maps-for-wordpress'); ?>:<br/>
				<input type="text" style='width:100%' name="<?php echo $this->get_field_name('mapid'); ?>" value="<?php echo $args->mapid ?>" />
			</p>

			<p>
				<?php _e('Map size', 'mappress-google-maps-for-wordpress'); ?>:
				<input size="3" id="<?php echo $this->get_field_id('width'); ?>" name="<?php echo $this->get_field_name('width'); ?>" type="text" value="<?php echo $args->width; ?>" />
				x <input size="3" id="<?php echo $this->get_field_id('height'); ?>" name="<?php echo $this->get_field_name('height'); ?>" type="text" value="<?php echo $args->height; ?>" />
			</p>

			<p>
				<label>
					<input type="checkbox" name="<?php echo $this->get_field_name('poiList'); ?>" <?php checked($args->poiList); ?> />
					<?php _e('Show POI list', 'mappress-google-maps-for-wordpress');?>
				</label>
				<br/>
			</p>

			<p>
				<br/>
				<?php _e('Other Settings', 'mappress-google-maps-for-wordpress'); ?>:<br/>
				<input type="text" style='width:100%' name="<?php echo $this->get_field_name('other'); ?>" value="<?php echo esc_attr($args->other); ?>" />
				<br/>
				<i><?php echo __('Example: initialopeninfo="true"', 'mappress-google-maps-for-wordpress');?></i>
			</p>
		<?php
	}
}
?>