<?php
namespace App\View\Composers;

use Roots\Acorn\View\Composer;

require_once get_template_directory() . '/app/helpers.php';

class Evento extends Composer {
    
    protected static $views = [
        'partials.content-single-evento',
        'blocks.map-advanced',
        'blocks.evento-in-evidenza',
    ];

    public function with() {
        return [
            'evento_singolo' => $this->get_evento_post()['evento_singolo'],
            'eventi_filtered' => $this->get_eventi_filtered()['events_filtered'],
            'totalpage' => $this->get_eventi_filtered()['totalpage'],
            'totalpost' => $this->get_eventi_filtered()['totalpost'],
            'currentpage' => $this->get_eventi_filtered()['currentpage'],
            'posts_per_page' => $this->get_eventi_filtered()['posts_per_page'],
        ];
    }

    public function get_evento_post() {
        $posts = default_query_giovanisi('evento', 1, 'ASC', array(), array(), '');
        $evento = [];
        $data_evento = calculateDateEvent($posts[0]->ID);
        $category_names = get_all_other_taxonomies($posts[0]->ID);
        $areetematiche = get_area_tematica($posts[0]->ID);
        $catevent = get_categoria_evento($posts[0]->ID);
        $cittaevento_acf = get_field('evento_citta', $posts[0]->ID);
        $cittaevento = isset($cittaevento_acf) && !empty($cittaevento_acf) ? $cittaevento_acf : '';
        $indirizzoevento_acf = get_field('evento_indirizzo', $posts[0]->ID);
        $indirizzoevento = isset($indirizzoevento_acf) && !empty($indirizzoevento_acf) ? $indirizzoevento_acf : '';
        $visibilita = get_field('visibilita_orario_evento', $posts[0]->ID);
        if(isset($visibilita) && $visibilita){
            $orarioevento = $data_evento['start']['hour'].':'.$data_evento['start']['minute'];
        } else {
            $orarioevento = '';
        }
        $evento_single = [];
        $evento[] = [
            'title' => $posts[0]->post_title,
            'permalink' => get_permalink($posts[0]->ID),
            'evento_data' => $data_evento,
            'categories' => $category_names,
            'aree_tematiche' => $areetematiche,
            'categoria' => $catevent,
            'cittaevento' => $cittaevento,
            'indirizzoevento' => $indirizzoevento,
            'orario' => $orarioevento
        ];
        $evento_single = $evento;   
        return [
            'evento_singolo' => $evento_single,
        ];
    }

    public function get_eventi_filtered() {
        $posts_per_page = 6;
        $current_page = 1;

        $taxonomy_filtered = get_field('seleziona_tipologia_di_evento_tab2_mappa');
        $terms_scelto = isset($taxonomy_filtered) && !empty($taxonomy_filtered) && $taxonomy_filtered != 'all' ? $taxonomy_filtered : '';

        $mostrafuturi_list = get_field('mostra_solo_futuri_tab2_mappa');
        $mostrasolofuturi_list = isset($mostrafuturi_list) && !empty($mostrafuturi_list) ? $mostrafuturi_list : false; // true/false
        $current_timestamp = time();

        $tax_query = [];
        if(!empty($terms_scelto)){
            $tax_query[] = [
                'taxonomy' => 'categoria-evento',
                'field' => 'slug',
                'terms' => $terms_scelto,
                'compare' => '='
            ];
        }

        $scelta_ordine = get_field('ordine_di_visualizzazione_tab2_mappa');
        $order = isset($scelta_ordine) && !empty($scelta_ordine) ? $scelta_ordine : 'ASC';
        $orderby = $order != 'ASC' ? 'menu_order' : '';

        if (is_front_page() && is_home()) {
            $current_page = get_query_var('paged') ? get_query_var('paged') : 1;
        } elseif (is_front_page()) {
            $current_page = get_query_var('page') ? get_query_var('page') : 1;
        } else {
            $current_page = get_query_var('paged') ? get_query_var('paged') : 1;
        }

        $meta_query = [];
        if ($mostrasolofuturi_list) {
            $meta_query = array(
                array('key' => 'evento_data_inizio', 'value' => date('Y-m-d H:i:s', $current_timestamp), 'compare' => '>', 'type' => 'DATETIME'),
            );
        }

        $posts = default_query_giovanisi('evento', -1, $order, $meta_query, $tax_query, '', $orderby, array(), '');

        usort($posts, function($a, $b) {
            $scelta_ordine = get_field('ordine_di_visualizzazione_tab2_mappa');
            $order = isset($scelta_ordine) && !empty($scelta_ordine) ? $scelta_ordine : 'ASC';
            $start_date_a = get_field('evento_data_inizio', $a->ID);
            $start_date_b = get_field('evento_data_inizio', $b->ID);
            $date_time_a = \DateTime::createFromFormat('d/m/Y H:i', $start_date_a);
            $date_time_b = \DateTime::createFromFormat('d/m/Y H:i', $start_date_b);
        
            // Gestione delle date non valide: assegna un timestamp molto alto per spostarle alla fine
            $timestamp_a = $date_time_a ? $date_time_a->getTimestamp() : 0;
            $timestamp_b = $date_time_b ? $date_time_b->getTimestamp() : 0;
            if($order != "DESC"){
                return $timestamp_b <=> $timestamp_a;
            } else {
                return $timestamp_a <=> $timestamp_b;
            }
        });

        $current_page = 1;

        $total_posts = count($posts);
        $totalPages = ceil($total_posts / $posts_per_page);
    
        // Calcola l'offset per la paginazione
        $offset = ($current_page - 1) * $posts_per_page;
        $posts = array_slice($posts, $offset, $posts_per_page);

        $tutti_gli_eventi = [];
        foreach ($posts as $post) {
            $data_evento = calculateDateEvent($post->ID);
            $category_names = get_all_other_taxonomies($post->ID);
            $areetematiche = get_area_tematica($post->ID);
            $catevent = get_categoria_evento($post->ID);
            $cittaevento_acf = get_field('evento_citta', $post->ID);
            $cittaevento = isset($cittaevento_acf) && !empty($cittaevento_acf) ? $cittaevento_acf : '';
            $indirizzoevento_acf = get_field('evento_indirizzo', $post->ID);
            $indirizzoevento = isset($indirizzoevento_acf) && !empty($indirizzoevento_acf) ? $indirizzoevento_acf : '';
            $visibilita = get_field('visibilita_orario_evento', $post->ID);
            if(isset($visibilita) && $visibilita){
                $orarioevento = $data_evento['start']['hour'].':'.$data_evento['start']['minute'];
            } else {
                $orarioevento = '';
            }
            $tutti_gli_eventi[] = [
                'title' => $post->post_title,
                'permalink' => get_permalink($post->ID),
                'evento_data' => $data_evento,
                'categories' => $category_names,
                'aree_tematiche' => $areetematiche,
                'categoria' => $catevent,
                'cittaevento' => $cittaevento,
                'indirizzoevento' => $indirizzoevento,
                'orario' => $orarioevento,
            ];
        } 
        $events_filtered = $tutti_gli_eventi;   
        return [
            'events_filtered' => $events_filtered,
            'totalpage' => $totalPages,
            'totalpost' => $total_posts,
            'currentpage' => 1,
            'posts_per_page' => $posts_per_page,
        ];
    }
}