<?php
namespace App\View\Composers;

use Roots\Acorn\View\Composer;

require_once get_template_directory() . '/app/helpers.php';

class FutureEvents extends Composer {
    
    protected static $views = [
        'blocks.calendario-eventi',
    ];

    public function with() {
        return [
            'futureEvents' => $this->get_filter_future_eventi_posts()['eventi_futuri'],
        ];
    }

    public function get_filter_future_eventi_posts() {
        
        $terms_area_tematica = get_field('scelta_area_tematica_blocco_calendario_eventi');
        $terms_categoria = get_field('scelta_categoria_blocco_calendario_eventi');
        $terms_tipologia = get_field('scelta_tipologia_blocco_calendario_eventi');

        $tax_query = [];

        if(!empty($terms_area_tematica)){
            $tax_query[] = [
                'taxonomy' => 'area-tematica',
                'field' => 'slug',
                'terms' => $terms_area_tematica,
                'compare' => '='
            ];
        }
        if(!empty($terms_categoria)){
            $tax_query[] = [
                'taxonomy' => 'categoria-evento',
                'field' => 'slug',
                'terms' => $terms_categoria,
                'compare' => '='
            ];
        }
        if(!empty($terms_tipologia)){
            $tax_query[] = [
                'taxonomy' => 'tipologia-evento',
                'field' => 'slug',
                'terms' => $terms_tipologia,
                'compare' => '='
            ];
        }

        $posts_future = default_query_giovanisi('evento', -1, 'ASC', array(), $tax_query, '');
        $eventi_futuri = [];
        $current_timestamp = time();
        foreach ($posts_future as $post) {
            $data_evento = calculateDateEvent($post->ID);
            $category_names = get_all_other_taxonomies($post->ID);
            $areetematiche = get_area_tematica($post->ID);
            $catevent = get_categoria_evento($post->ID);
            $start_date = get_field('evento_data_inizio', $post->ID);
            $cittaevento_acf = get_field('evento_citta', $post->ID);
            $cittaevento = isset($cittaevento_acf) && !empty($cittaevento_acf) ? $cittaevento_acf : '';
            $indirizzoevento_acf = get_field('evento_indirizzo', $post->ID);
            $indirizzoevento = isset($indirizzoevento_acf) && !empty($indirizzoevento_acf) ? $indirizzoevento_acf : '';
            $visibilita = get_field('visibilita_orario_evento', $post->ID);
            if(isset($visibilita) && $visibilita){
                $orarioevento = $data_evento['start']['hour'].':'.$data_evento['start']['minute'];
            } else {
                $orarioevento = '';
            }
            if (!empty($start_date)) {
                $date_time = \DateTime::createFromFormat('d/m/Y H:i', $start_date);
                if ($date_time !== false) {
                    $data_ev = $date_time->getTimestamp();
                    if ($current_timestamp < $data_ev) {
                        $eventi_futuri[] = [
                            'title' => $post->post_title,
                            'permalink' => get_permalink($post),
                            'evento_data' => $data_evento,
                            'categories' => $category_names,
                            'aree_tematiche' => $areetematiche,
                            'categoria' => $catevent,
                            'data_inizio_timestamp' => $data_ev,
                            'cittaevento' => $cittaevento,
                            'indirizzoevento' => $indirizzoevento,
                            'orario' => $orarioevento,
                        ];
                    }
                }
            }
        }
        
        usort($eventi_futuri, function($a, $b) {
            return $b['data_inizio_timestamp'] <=> $a['data_inizio_timestamp'];
        });

        $filtro_asc_desc = get_field('ordine_di_visualizzazione_blocco_calendario_eventi');
        if(isset($filtro_asc_desc) && !empty($filtro_asc_desc) && $filtro_asc_desc == 'asc'){
            $eventi_futuri = array_reverse($eventi_futuri);
        }

        return [
            'eventi_futuri' => $eventi_futuri,
        ];
    }
}