<?php

namespace App\View\Composers;

use Roots\Acorn\View\Composer;

require_once get_template_directory() . '/app/helpers.php';

class Storie extends Composer
{

    protected static $views = [
        'blocks.storiepossibili',
        'partials.content-single-storia',
        'blocks.filter-storie'
    ];

    public function with(){
        return [
            'storie_post' => $this->getStoriePosts()['storie'],
            'tipologia' => $this->getStoriePosts()['tipologia'],
            'totalpage' => $this->getStoriePosts()['totalpage'],
            'totalpost' => $this->getStoriePosts()['totalpost'],
            'currentpage' => $this->getStoriePosts()['currentpage'],
        ];
    }

    public function getStoriePosts(){

        //Scelta storie da visualizzare - usata da Blocco Storie Possibili
        $selectedPosts = get_field('storie_possibili_scelta_storie');

        //Scelta visualizzazione card - usata da Blocco Storie Possibili
        $visual = get_field('storie_possibili_scelta_tipologia_card');
        $visualcard = null;
        $card_to_show = -1;
        if (isset($visual)) {
            if ($visual === 'tipo_2' || $visual === 'tipo_1') {
                $visualcard = $visual;
                $card_to_show = 10;
            }
        }

        //Area tematica di dafault - usata da Blocco Storie Possibili
        $areatematica = get_field('scelta_area_tematica_default_storie_possibili');
        $areatematica_default = isset($areatematica->slug) && !empty($areatematica->slug) ? $areatematica->slug : '';

        // Se selezionati prendi quelli
        if ($selectedPosts && is_array($selectedPosts) && !empty($selectedPosts)) {
            $posts = $visualcard === 'tipo_2' ? array_slice($selectedPosts, 0, 2) : $selectedPosts;
        }

        //Se impostata un'area tematica di default prendi quella
        elseif (!empty($areatematica_default)){
            $posts = default_query_giovanisi('storia', $card_to_show, 'ASC',[],[['taxonomy' => 'area-tematica', 'field' => 'slug', 'terms' => $areatematica_default]]);
            //Solo se non siamo nel listato (Blocco Filtri Storie)
            if ($card_to_show !== -1) {
                shuffle($posts);
                //Se visual card è di tipo 2 (con titolo e testo) mostrane 2 altrimenti 4
                $posts = array_slice($posts, 0, $visualcard === 'tipo_2' ? 2 : 4);
            }
        }

        //Se siamo nel singolo della storia
        elseif (is_singular('storia')) {
            $terms_corrente = !empty(get_the_terms(get_the_id(), 'area-tematica')[0]->slug) && isset(get_the_terms(get_the_id(), 'area-tematica')[0]->slug) ? get_the_terms(get_the_id(), 'area-tematica')[0]->slug : '';
            $posts = default_query_giovanisi('storia', 10, 'ASC',[],[['taxonomy' => 'area-tematica', 'field' => 'slug', 'terms' => $terms_corrente ]], '', '', [get_the_id()]);
            shuffle($posts);
            $posts = array_slice($posts, 0, 4);
        }
        //Se non sono stati selezionati articoli specifici e non è stata impostata un'area tematica, prendi tutte le storie
        else {
            $posts = default_query_giovanisi('storia', $card_to_show, 'ASC',[],[]);
            //Solo se non siamo nel listato (Blocco Filtri Storie)
            if ($card_to_show !== -1) {

                $posts = [];
                //Uno random degli ultimi 10 per ognuna delle 4 aree tematiche 
                $storie_studio [] = default_query_giovanisi('storia', 10, 'ASC', array(), array( array('taxonomy' => 'area-tematica', 'field' => 'slug', 'terms' => 'studio-e-mi-formo',)), null, 'menu_order');
                $storie_partecipo [] = default_query_giovanisi('storia', 10, 'ASC', array(), array( array('taxonomy' => 'area-tematica', 'field' => 'slug', 'terms' => 'partecipo',)), null, 'menu_order');
                $storie_impresa [] = default_query_giovanisi('storia', 10, 'ASC', array(), array( array('taxonomy' => 'area-tematica', 'field' => 'slug', 'terms' => 'faccio-impresa',)), null, 'menu_order');
                $storie_lavoro [] = default_query_giovanisi('storia', 10, 'ASC', array(), array( array('taxonomy' => 'area-tematica', 'field' => 'slug', 'terms' => 'lavoro',)), null, 'menu_order');
                
                if(!empty($storie_studio)){
                    shuffle($storie_studio[0]);
                    $posts [] = $storie_studio[0][0];
                }
                if(!empty($storie_partecipo)){
                    shuffle($storie_partecipo[0]);
                    $posts [] = $storie_partecipo[0][0];
                }
                if(!empty($storie_impresa)){
                    shuffle($storie_impresa[0]);
                    $posts [] = $storie_impresa[0][0];
                }
                if(!empty($storie_lavoro)){
                    shuffle($storie_lavoro[0]);
                    $posts [] = $storie_lavoro[0][0];
                }
            }

        }

        $posts_per_page = 8;
        $current_page = 1;

        $total_posts = count($posts);
        $totalPages = ceil($total_posts / $posts_per_page);
    
        // Calcola l'offset per la paginazione
        $offset = ($current_page - 1) * $posts_per_page;
        $posts = array_slice($posts, $offset, $posts_per_page);
        // Ritorna un array contenente sia le storie che la tipologia
        return [
            'storie' => formatPostsStoria($posts),
            'tipologia' => $visualcard,
            'totalpage' => $totalPages,
            'totalpost' => $total_posts,
            'currentpage' => 1
        ];
    }
    
}
