<?php
namespace App\Http;

class Ajax {
    public static function charge_bandi() {
        // Verifica il nonce per sicurezza
        check_ajax_referer('my_nonce', 'nonce');

        $is_archive = isset($_POST['isarchive']) ? $_POST['isarchive'] : 'no';
        $posts_per_page = isset($_POST['postperpage']) ? $_POST['postperpage'] : 8;
        $status = isset($_POST['status']) ? sanitize_text_field($_POST['status']) : '';
        $eta = isset($_POST['eta']) ? sanitize_text_field($_POST['eta']) : '';
        $page = isset($_POST['page']) ? intval($_POST['page']) : 1;
        $area_tematica = isset($_POST['area_tematica']) ? sanitize_text_field($_POST['area_tematica']) : '';
        $meta_query = [];
        if($is_archive == 'si'){
            $meta_query[] = [
                'key' => 'stato',
                'value' => 'archivio',
                'compare' => '='
            ];
        } else {
            if (!empty($status)) {
                $meta_query[] = [
                    'key' => 'stato',
                    'value' => $status,
                    'compare' => '='
                ];
            } else {
                $meta_query[] = [
                    'key' => 'stato',
                    'value' => 'archivio',
                    'compare' => '!='
                ];
            }
        }
        $tax_query = [];
        if (!empty($eta)) {
            $tax_query[] = [
                'taxonomy' => 'fascia-eta',
                'field' => 'slug',
                'terms' => $eta,
                'compare' => '='
            ];
        }
        if (!empty($area_tematica)) {
            $tax_query[] = [
                'taxonomy' => 'area-tematica',
                'field' => 'slug',
                'terms' => $area_tematica,
                'compare' => '='
            ];
        }

        $offset = ($page - 1) * $posts_per_page;

        $html = '';
        $bando_posts = default_query_giovanisi('bando', -1, 'ASC', $meta_query, $tax_query, $page);
        
        $total_posts = count($bando_posts);
        $total_pages = ceil($total_posts / $posts_per_page);

        $bando_posts = array_slice($bando_posts, $offset, $posts_per_page);
        $posts_with_categories = [];
        
        foreach ($bando_posts as $post) {
            $target_acf = get_field('target', $post->ID);
            $target = !empty($target_acf) && isset($target_acf) ? $target_acf : '';
            $categories = get_all_other_taxonomies($post->ID);
            $area_tematica = get_area_tematica($post->ID);
            $stato = check_stato_bando($post->ID);
            $post_data = [
                'post_id'          => $post->ID,
                'post_title'       => $post->post_title,
                'post_permalink'   => get_permalink($post->ID),
                'area_tematica'    => $area_tematica,
                'taxonomy'         => $categories,
                'stato'            => $stato,
                'target'           => $target,
            ];
            $posts_with_categories[] = $post_data;
        }

        $html .= view('partials.elenco-bandi', [
            'bando_all_posts' => $posts_with_categories,
            'totalpage' => $total_pages,
            'currentpage' => $page,
            'totalpost' => $total_posts,]
            )->render();

        if (empty($posts_with_categories)) {
            $html = '<div class="flex gap-6 items-center">
            <svg aria-hidden="true" width="72" height="70" fill="none"
                xmlns="http://www.w3.org/2000/svg">
                <rect width="72" height="70" rx="35" fill="#000" fill-opacity=".25"/>
                <path d="M59 44.13s-5.162-4.559-11.472-10.12c5.313-6.248 9.765-11.48 10.23-12.03C58.972 20.554 47.623 13 47.623 13L37.12 24.837c-6.377-5.63-11.8-10.409-12.36-10.903C23.356 12.684 14 22.79 14 22.79l13.794 12.565L16.28 48.332l11.881 8.473s4.562-5.37 10.107-11.906l12.154 11.068L59 44.13z" fill="#fff"/>
            </svg>
            <p class="text-base md:text-lg lg:text-xl">Nessun risultato trovato.</p>
            </div>';
        }
        echo $html;
        wp_die();
    }         

    public static function charge_storie() {
        // Verifica il nonce per sicurezza
        check_ajax_referer('my_nonce', 'nonce');

        $page = isset($_POST['page']) ? intval($_POST['page']) : 1;
        $area_tematica = isset($_POST['area_tematica']) ? sanitize_text_field($_POST['area_tematica']) : '';
        $tax_query = [];
        if (!empty($area_tematica)) {
            $tax_query[] = [
                'taxonomy' => 'area-tematica',
                'field' => 'slug',
                'terms' => $area_tematica,
                'compare' => '='
            ];
        }

        $html = '';
        $posts_per_page = 8;

        $html = '';
        $storie_posts = default_query_giovanisi('storia', -1, 'ASC', '', $tax_query, $page);

        $offset = ($page - 1) * $posts_per_page;
        $current_page = 1;

        
        $total_posts = count($storie_posts);
        $total_pages = ceil($total_posts / $posts_per_page);

        $storie_posts = array_slice($storie_posts, $offset, $posts_per_page);
        $storie_posts = formatPostsStoria($storie_posts);

        $html .= view('partials.wrapper-storie-possibili', [
            'storie_post' => $storie_posts,
            'totalpage' => $total_pages,
            'totalpost' => $total_posts,
            'currentpage' => $page,
            'classes_wrapper' => 'grid gap-4 md:grid-cols-2 lg:grid-cols-2',
            'include' => 'card-storiepossibili2',
            'area_visible' => true,
        ])->render();
        
        if (empty($storie_posts)) {
            $html = '<div class="flex gap-6 items-center">
            <svg aria-hidden="true" width="72" height="70" fill="none"
                xmlns="http://www.w3.org/2000/svg">
                <rect width="72" height="70" rx="35" fill="#000" fill-opacity=".25"/>
                <path d="M59 44.13s-5.162-4.559-11.472-10.12c5.313-6.248 9.765-11.48 10.23-12.03C58.972 20.554 47.623 13 47.623 13L37.12 24.837c-6.377-5.63-11.8-10.409-12.36-10.903C23.356 12.684 14 22.79 14 22.79l13.794 12.565L16.28 48.332l11.881 8.473s4.562-5.37 10.107-11.906l12.154 11.068L59 44.13z" fill="#fff"/>
            </svg>
            <p class="text-base md:text-lg lg:text-xl">Nessun risultato trovato.</p>
            </div>';
        }
        echo $html;
        wp_die();
    }

    public static function charge_category_news() {
        // Verifica il nonce per sicurezza
        check_ajax_referer('my_nonce', 'nonce');

        $page = isset($_POST['page']) ? intval($_POST['page']) : 1;
        $categoria = isset($_POST['categoria']) ? sanitize_text_field($_POST['categoria']) : '';
        $tax_query = [];
        $terms = get_terms('area-tematica', array('fields'=>'ids'));
        if (!empty($categoria)) {
            if($categoria == 'varie'){
                $tax_query[] = [
                    'relation' => 'OR',
                    array(
                        'taxonomy' => 'area-tematica',
                        'field'    => 'slug',
                        'terms'    => $terms,
                    ),
                    array(
                        'taxonomy' => 'area-tematica',
                        'operator' => 'NOT EXISTS',
                    ),
                ];
            } else {
                $tax_query[] = [
                    'taxonomy' => 'area-tematica',
                    'field' => 'slug',
                    'terms' => $categoria,
                    'compare' => '='
                ];
            }
        }

        $tax_query[] = array(
            'taxonomy' => 'category',
            'field' => 'slug',
            'terms' => 'senza-categoria',
            'operator' => 'NOT IN',
        );

        $html = '';
        $posts_per_page = 12;

        $html = '';
        $news_posts = default_query_giovanisi('post', -1, 'DESC', '', $tax_query, $page, '');

        $offset = ($page - 1) * $posts_per_page;
        $current_page = 1;
        
        $total_posts = count($news_posts);
        $total_pages = ceil($total_posts / $posts_per_page);

        $news_posts = array_slice($news_posts, $offset, $posts_per_page);

        $html .= view('partials.elenco-news', [
            'post' => $news_posts,
            'totalpage' => $total_pages,
            'totalpost' => $total_posts,
            'currentpage' => $page
        ])->render();
        
        if (empty($news_posts)) {
            $html = '<div class="flex gap-6 items-center">
            <svg aria-hidden="true" width="72" height="70" fill="none"
                xmlns="http://www.w3.org/2000/svg">
                <rect width="72" height="70" rx="35" fill="#000" fill-opacity=".25"/>
                <path d="M59 44.13s-5.162-4.559-11.472-10.12c5.313-6.248 9.765-11.48 10.23-12.03C58.972 20.554 47.623 13 47.623 13L37.12 24.837c-6.377-5.63-11.8-10.409-12.36-10.903C23.356 12.684 14 22.79 14 22.79l13.794 12.565L16.28 48.332l11.881 8.473s4.562-5.37 10.107-11.906l12.154 11.068L59 44.13z" fill="#fff"/>
            </svg>
            <p class="text-base md:text-lg lg:text-xl">Nessun risultato trovato.</p>
            </div>';
        }
        echo $html;
        wp_die();
    }

    public static function charge_campagne() {
        // Verifica il nonce per sicurezza
        check_ajax_referer('my_nonce', 'nonce');

        $page = isset($_POST['page']) ? intval($_POST['page']) : 1;
        $evidenza_id = isset($_POST['evidenza']) ? $_POST['evidenza'] : '';

        // Usa explode() per dividere la stringa in un array basato sul separatore '-'
        $parts = explode('-', $evidenza_id);
        $evidenza_id_formatted = [];
        
        foreach ($parts as $part) {
            if (!empty($part)) {
                $evidenza_id_formatted[] = (int) $part; // Cast
            }
        }

        $html = '';
        $posts_per_page = 8;

        $all_campagne = default_query_giovanisi('campagna', -1, 'ASC', array(), array(), '', '', $evidenza_id_formatted);

        $offset = ($page - 1) * $posts_per_page;
        $current_page = 1;
        
        $total_posts = count($all_campagne);
        $total_pages = ceil($total_posts / $posts_per_page);

        $all_campagne = array_slice($all_campagne, $offset, $posts_per_page);

        $html .= view('partials.elenco-campagne', [
            'campagne' => $all_campagne,
            'totalpage' => $total_pages,
            'totalpost' => $total_posts,
            'currentpage' => $page
        ])->render();
        
        if (empty($all_campagne)) {
            $html = '<div class="flex gap-6 items-center">
            <svg aria-hidden="true" width="72" height="70" fill="none"
                xmlns="http://www.w3.org/2000/svg">
                <rect width="72" height="70" rx="35" fill="#000" fill-opacity=".25"/>
                <path d="M59 44.13s-5.162-4.559-11.472-10.12c5.313-6.248 9.765-11.48 10.23-12.03C58.972 20.554 47.623 13 47.623 13L37.12 24.837c-6.377-5.63-11.8-10.409-12.36-10.903C23.356 12.684 14 22.79 14 22.79l13.794 12.565L16.28 48.332l11.881 8.473s4.562-5.37 10.107-11.906l12.154 11.068L59 44.13z" fill="#fff"/>
            </svg>
            <p class="text-base md:text-lg lg:text-xl">Nessun risultato trovato.</p>
            </div>';
        }
        echo $html;
        wp_die();
    }

    public static function charge_postmagazine() {
        // Verifica il nonce per sicurezza
        check_ajax_referer('my_nonce', 'nonce');

        $page = isset($_POST['page']) ? intval($_POST['page']) : 1;

        $html = '';
        $posts_per_page = 12;

        $all_postmagazine = default_query_giovanisi('post-magazine', -1, 'DESC', array(), array(), '');

        $offset = ($page - 1) * $posts_per_page;
        $current_page = 1;
        
        $total_posts = count($all_postmagazine);
        $total_pages = ceil($total_posts / $posts_per_page);

        $all_postmagazine = array_slice($all_postmagazine, $offset, $posts_per_page);

        $html .= view('partials.elenco-postmagazine', [
            'post' => $all_postmagazine,
            'totalpage' => $total_pages,
            'totalpost' => $total_posts,
            'currentpage' => $page
        ])->render();
        
        if (empty($all_postmagazine)) {
            $html = '<div class="flex gap-6 items-center">
            <svg aria-hidden="true" width="72" height="70" fill="none"
                xmlns="http://www.w3.org/2000/svg">
                <rect width="72" height="70" rx="35" fill="#000" fill-opacity=".25"/>
                <path d="M59 44.13s-5.162-4.559-11.472-10.12c5.313-6.248 9.765-11.48 10.23-12.03C58.972 20.554 47.623 13 47.623 13L37.12 24.837c-6.377-5.63-11.8-10.409-12.36-10.903C23.356 12.684 14 22.79 14 22.79l13.794 12.565L16.28 48.332l11.881 8.473s4.562-5.37 10.107-11.906l12.154 11.068L59 44.13z" fill="#fff"/>
            </svg>
            <p class="text-base md:text-lg lg:text-xl">Nessun risultato trovato.</p>
            </div>';
        }
        echo $html;
        wp_die();
    }

    public static function charge_searchresults() {
        // Verifica il nonce per sicurezza
        check_ajax_referer('my_nonce', 'nonce');

        $page = isset($_POST['page']) ? intval($_POST['page']) : 1;
        $s = isset($_POST['searchkey']) ? $_POST['searchkey'] : '';

        $html = '';
        $posts_per_page = 12;

        $post_type_to_search = ['post', 'bando', 'page' , 'storia', 'post-magazine', 'evento', 'campagna', 'faq'];

        $searchpost = default_query_giovanisi($post_type_to_search, -1, 'ASC', null, null, null, null, null, $s);

        $current_page = $page;

        $total_posts = count($searchpost);
        $totalPages = ceil($total_posts / $posts_per_page);
    
        // Calcola l'offset per la paginazione
        $offset = ($current_page - 1) * $posts_per_page;
        $searchpost = array_slice($searchpost, $offset, $posts_per_page);

        $html .= view('partials.elenco-search', [
            'search_post' => $searchpost,
            'totalpage' => $totalPages,
            'totalpost' => $total_posts,
            'currentpage' => $page
        ])->render();
        
        if (empty($searchpost)) {
            $html = '<div class="flex gap-6 items-center">
            <svg aria-hidden="true" width="72" height="70" fill="none"
                xmlns="http://www.w3.org/2000/svg">
                <rect width="72" height="70" rx="35" fill="#000" fill-opacity=".25"/>
                <path d="M59 44.13s-5.162-4.559-11.472-10.12c5.313-6.248 9.765-11.48 10.23-12.03C58.972 20.554 47.623 13 47.623 13L37.12 24.837c-6.377-5.63-11.8-10.409-12.36-10.903C23.356 12.684 14 22.79 14 22.79l13.794 12.565L16.28 48.332l11.881 8.473s4.562-5.37 10.107-11.906l12.154 11.068L59 44.13z" fill="#fff"/>
            </svg>
            <p class="text-base md:text-lg lg:text-xl">Nessun risultato trovato.</p>
            </div>';
        }
        echo $html;
        wp_die();
    }


    public static function charge_eventi_filtered() {
        // Verifica il nonce per sicurezza
        check_ajax_referer('my_nonce', 'nonce');

        $page = isset($_POST['page']) ? intval($_POST['page']) : 1;
        $categoria = isset($_POST['categoria']) ? $_POST['categoria'] : '';
        $order = isset($_POST['order']) ? $_POST['order'] : 'ASC';
        $orderby = $order != 'ASC' ? 'menu_order' : '';
        $futuri = isset($_POST['futuri']) ? $_POST['futuri'] : 'yes';
        
        $html = '';
        $posts_per_page = 6;
        $current_timestamp = time();

        if(!empty($categoria)){
            $tax_query[] = [
                'taxonomy' => 'categoria-evento',
                'field' => 'slug',
                'terms' => $categoria,
                'compare' => '='
            ];
        } else {
            $tax_query = [];
        }

        $meta_query = [];
        if ($futuri) {
            // Mostra tutti gli eventi
            $meta_query = array(
                array('key' => 'evento_data_inizio', 'value' => date('Y-m-d H:i:s', $current_timestamp), 'compare' => '>', 'type' => 'DATETIME'),
            );
        }

        //$all_eventi = default_query_giovanisi('evento', -1, $order, array(), $tax_query, '');
        $all_eventi = default_query_giovanisi('evento', -1, $order, $meta_query, $tax_query, '', $orderby, array(), '');

        if($order != 'DESC'){
            usort($all_eventi, function($a, $b) {
                $start_date_a = get_field('evento_data_inizio', $a->ID);
                $start_date_b = get_field('evento_data_inizio', $b->ID);
                $date_time_a = \DateTime::createFromFormat('d/m/Y H:i', $start_date_a);
                $date_time_b = \DateTime::createFromFormat('d/m/Y H:i', $start_date_b);
            
                // Gestione delle date non valide: assegna un timestamp molto alto per spostarle alla fine
                $timestamp_a = $date_time_a ? $date_time_a->getTimestamp() : 0;
                $timestamp_b = $date_time_b ? $date_time_b->getTimestamp() : 0;
            
                return $timestamp_b <=> $timestamp_a;
            });
        } else {
            usort($all_eventi, function($a, $b) {
                $start_date_a = get_field('evento_data_inizio', $a->ID);
                $start_date_b = get_field('evento_data_inizio', $b->ID);
                $date_time_a = \DateTime::createFromFormat('d/m/Y H:i', $start_date_a);
                $date_time_b = \DateTime::createFromFormat('d/m/Y H:i', $start_date_b);
            
                // Gestione delle date non valide: assegna un timestamp molto alto per spostarle alla fine
                $timestamp_a = $date_time_a ? $date_time_a->getTimestamp() : 0;
                $timestamp_b = $date_time_b ? $date_time_b->getTimestamp() : 0;
            
                return $timestamp_a <=> $timestamp_b;
            });
        }

        $offset = ($page - 1) * $posts_per_page;
        $current_page = $page;
        
        $total_posts = count($all_eventi);
        $total_pages = ceil($total_posts / $posts_per_page);

        $all_eventi = array_slice($all_eventi, $offset, $posts_per_page);

        $tutti_gli_eventi = [];
        $events_filtered = [];
        foreach ($all_eventi as $post) {
            $data_evento = calculateDateEvent($post->ID);
            $category_names = get_all_other_taxonomies($post->ID);
            $areetematiche = get_area_tematica($post->ID);
            $catevent = get_categoria_evento($post->ID);
            $cittaevento_acf = get_field('evento_citta', $post->ID);
            $cittaevento = isset($cittaevento_acf) && !empty($cittaevento_acf) ? $cittaevento_acf : '';
            $indirizzoevento_acf = get_field('evento_indirizzo', $post->ID);
            $indirizzoevento = isset($indirizzoevento_acf) && !empty($indirizzoevento_acf) ? $indirizzoevento_acf : '';
            $visibilita = get_field('visibilita_orario_evento', $post->ID);
            if(isset($visibilita) && $visibilita){
                $orarioevento = $data_evento['start']['hour'].':'.$data_evento['start']['minute'];
            } else {
                $orarioevento = '';
            }
            $tutti_gli_eventi[] = [
                'title' => $post->post_title,
                'permalink' => get_permalink($post->ID),
                'evento_data' => $data_evento,
                'categories' => $category_names,
                'aree_tematiche' => $areetematiche,
                'categoria' => $catevent,
                'futuri' => $futuri,
                'cittaevento' => $cittaevento,
                'indirizzoevento' => $indirizzoevento,
                'orario' => $orarioevento
            ];
        } 
        $events_filtered = $tutti_gli_eventi;   

        $html .= view('partials.elenco-eventi', [
            'eventi_filtered' => $events_filtered,
            'totalpage' => $total_pages,
            'totalpost' => $total_posts,
            'currentpage' => $page,
            'posts_per_page' => $posts_per_page,
        ])->render();
        
        if (empty($all_eventi)) {
            $html = '<div class="flex gap-6 items-center">
            <svg aria-hidden="true" width="72" height="70" fill="none"
                xmlns="http://www.w3.org/2000/svg">
                <rect width="72" height="70" rx="35" fill="#000" fill-opacity=".25"/>
                <path d="M59 44.13s-5.162-4.559-11.472-10.12c5.313-6.248 9.765-11.48 10.23-12.03C58.972 20.554 47.623 13 47.623 13L37.12 24.837c-6.377-5.63-11.8-10.409-12.36-10.903C23.356 12.684 14 22.79 14 22.79l13.794 12.565L16.28 48.332l11.881 8.473s4.562-5.37 10.107-11.906l12.154 11.068L59 44.13z" fill="#fff"/>
            </svg>
            <p class="text-base md:text-lg lg:text-xl">Nessun risultato trovato.</p>
            </div>';
        }
        echo $html;
        wp_die();
    }

    public static function carica_mappa() {
        // Verifica il nonce per sicurezza
        check_ajax_referer('my_nonce', 'nonce');
        $html = do_shortcode('[mappress mapid="2"]');
        echo $html;
        wp_die();
    }
}
