<?php

use Illuminate\Support\HtmlString;

function calculateDateEvent($post_id) {
    // Recupera le date di inizio e fine dall'ACF
    $start_date_string = get_field('evento_data_inizio', $post_id);
    $end_date_string = get_field('evento_data_fine', $post_id);

    // Inizializza l'array per la data di inizio e di fine
    $event_dates = [];
    $event_dates['display'] = '';
    $event_dates['start'] = []; 
    $event_dates['end'] = []; 
    
    // Verifica se la data di inizio è stata fornita e non è vuota
    if (!empty($start_date_string)) {
        $start_date = DateTime::createFromFormat('d/m/Y H:i', $start_date_string);

        $formatterstart = new IntlDateFormatter(
            'it_IT',
            IntlDateFormatter::NONE,
            IntlDateFormatter::NONE,
            $start_date->getTimezone(),
            IntlDateFormatter::GREGORIAN,
            'MMMM'
        );

        $event_dates['start'] = [
            'hour' => $start_date->format('H'),
            'minute' => $start_date->format('i'),
            'day' => ltrim($start_date->format('d'), '0'),
            'month' => $formatterstart->format($start_date),
            'monthnum' => $start_date->format('m'),
            'year' => $start_date->format('Y')
        ];
    }

    // Verifica se la data di fine è stata fornita e non è vuota
    if (!empty($end_date_string)) {
        $end_date = DateTime::createFromFormat('d/m/Y H:i', $end_date_string);

        $formatterend = new IntlDateFormatter(
            'it_IT',
            IntlDateFormatter::NONE,
            IntlDateFormatter::NONE,
            $end_date->getTimezone(),
            IntlDateFormatter::GREGORIAN,
            'MMMM'
        );

        $event_dates['end'] = [
            'hour' => $end_date->format('H'),
            'minute' => $end_date->format('i'),
            'day' => ltrim($end_date->format('d'), '0'),
            'month' => $formatterend->format($end_date),
            'monthnum' => $end_date->format('m'),
            'year' => $end_date->format('Y')
        ];
    }

    // Se inserita solo la data di inizio, restituisci l'array delle date
    if (!empty($event_dates['start']) && (!isset($event_dates['end']) || empty($event_dates['end']))) {
        // Verifica se sia l'ora che i minuti sono entrambi 00
        if ($event_dates['start']['hour'] !== '00' || $event_dates['start']['minute'] !== '00') {
            // Stampa l'ora solo se entrambi non sono 00
            $event_dates['display'] =  
            // '<time class="grid gap-1" datetime="2024-10-12"><div class="evento-ora">'.$event_dates['start']['hour'].':'.$event_dates['start']['minute'].'</div><div class="block font-serif text-6xl">'.$event_dates['start']['day'].'</div><div class="italic font-bold text-2xl leading-none">'.$event_dates['start']['month'].'</div><div class="text-sm">'.$event_dates['start']['year'].'</div></time>';
            '<time class="grid gap-1" datetime="'.$event_dates['start']['year'].'-'.$event_dates['start']['monthnum'].'-'.$event_dates['start']['day'].'"><div class="block font-serif text-6xl">'.$event_dates['start']['day'].'</div><div class="italic font-bold text-2xl leading-none">'.$event_dates['start']['month'].'</div><div class="text-sm leading-none">'.$event_dates['start']['year'].'</div></time>';
        } else {
            // Non stampare l'ora se entrambi sono 00
            $event_dates['display'] =  
            '<time class="grid gap-1" datetime="'.$event_dates['start']['year'].'-'.$event_dates['start']['monthnum'].'-'.$event_dates['start']['day'].'"><div class="block font-serif text-6xl">'.$event_dates['start']['day'].'</div><div class="italic font-bold text-2xl leading-none">'.$event_dates['start']['month'].'</div><div class="text-sm">'.$event_dates['start']['year'].'</div></time>';
        }

        return $event_dates;
    }

    // Se inserite entrambe le date
    elseif(!empty($event_dates['start']) && !empty($event_dates['end'])){

        // Se le date sono uguali
        if($event_dates['start'] == $event_dates['end']){
            $event_dates['display'] =  
            '<time class="grid gap-1" datetime="'.$event_dates['start']['year'].$event_dates['start']['monthnum'].$event_dates['start']['day'].'"><div class="block font-serif text-6xl">'.$event_dates['start']['day'].'</div><div class="italic font-bold text-2xl leading-none">'.$event_dates['start']['month'].'</div><div class="text-sm">'.$event_dates['start']['year'].'</div></time>';
            return $event_dates;

        // Se le date sono diverse
        } else {

            // Se l'anno è uguale
            if ($event_dates['start']['year'] == $event_dates['end']['year']) {

                // Se anche il mese è uguale
                if ($event_dates['start']['month'] == $event_dates['end']['month']) {

                    // Se anche il giorno è uguale
                    if ($event_dates['start']['day'] == $event_dates['end']['day']) {

                        $event_dates['display'] = 
                        // '<time class="grid gap-1" datetime="2024-10-12"><div class="evento-ora">'.$event_dates['start']['hour'].':'.$event_dates['start']['minute'].'</div> / <div class="evento-ora">'.$event_dates['end']['hour'].':'.$event_dates['end']['minute'].'</div><div class="block font-serif text-6xl">'.$event_dates['start']['day'].'</div><div class="italic font-bold text-2xl leading-none">'.$event_dates['start']['month'].'</div><div class="text-sm">'.$event_dates['start']['year'].'</div></time>';
                        '<time class="grid gap-1" datetime="'.$event_dates['start']['year'].'-'.$event_dates['start']['monthnum'].'-'.$event_dates['start']['day'].'"><div class="block font-serif text-6xl">'.$event_dates['start']['day'].'</div><div class="italic font-bold text-2xl leading-none">'.$event_dates['start']['month'].'</div><div class="text-sm">'.$event_dates['start']['year'].'</div></time>';
                        return $event_dates;
                    
                    // Se anno e mese sono uguali ma giorno diverso
                    } else {

                        $event_dates['display'] =
                        '<time class="grid gap-1" datetime="'.$event_dates['start']['year'].'-'.$event_dates['start']['monthnum'].'-'.$event_dates['start']['day'].'/'.$event_dates['end']['year'].'-'.$event_dates['end']['monthnum'].'-'.$event_dates['end']['day'].'"><div class="block font-serif text-6xl">'.$event_dates['start']['day'].'/</div><div class="block font-serif text-6xl">'.$event_dates['end']['day'].'</div><div class="italic font-bold text-2xl leading-none">'.$event_dates['start']['month'].'</div><div class="text-sm">'.$event_dates['start']['year'].'</div></time>';
                        return $event_dates;
                    }

                // Se mese è diverso e l'anno è uguale
                } else{
                    
                    $event_dates['display'] =
                    '<time class="grid gap-1" datetime="'.$event_dates['start']['year'].'-'.$event_dates['start']['monthnum'].'-'.$event_dates['start']['day'].'/'.$event_dates['end']['year'].'-'.$event_dates['end']['monthnum'].'-'.$event_dates['end']['day'].'"><div class="block font-serif text-6xl">'.$event_dates['start']['day'].'</div><div class="italic font-bold text-2xl leading-none">'.$event_dates['start']['month'].' / </div><div class="block font-serif text-6xl">'.$event_dates['end']['day'].'</div><div class="italic font-bold text-2xl leading-none">'.$event_dates['end']['month'].'</div><div class="text-sm">'.$event_dates['start']['year'].'</div></time>';    
                    return $event_dates;
                }

            // Se è tutto diverso
            } else {
                $event_dates['display'] =
                '<time class="grid gap-1" datetime="'.$event_dates['start']['year'].'-'.$event_dates['start']['monthnum'].'-'.$event_dates['start']['day'].'/'.$event_dates['end']['year'].'-'.$event_dates['end']['monthnum'].'-'.$event_dates['end']['day'].'"><div class="block font-serif text-6xl">'.$event_dates['start']['day'].'</div><div class="italic font-bold text-2xl leading-none">'.$event_dates['start']['month'].'</div><div class="text-sm">'.$event_dates['start']['year'].' / </div><div class="block font-serif text-6xl">'.$event_dates['end']['day'].'</div><div class="italic font-bold text-2xl leading-none">'.$event_dates['end']['month'].'</div><div class="text-sm">'.$event_dates['end']['year'].'</div></time>';  
                return $event_dates;
            }
        }
    }

    // dd($event_dates);

    // Restituisci l'array con le date di inizio e fine, se disponibili
    return $event_dates;
}

function calculateDateEventNotFormatted($post_id) {
    // Recupera le date di inizio e fine dall'ACF
    $start_date_string = get_field('evento_data_inizio', $post_id);
    $end_date_string = get_field('evento_data_fine', $post_id);

    // Inizializza l'array per la data di inizio e di fine
    $event_dates = [];
    $event_dates['display'] = '';
    $event_dates['start'] = []; 
    $event_dates['end'] = []; 
    // Verifica se la data di inizio è stata fornita e non è vuota
    if (!empty($start_date_string)) {
        $start_date = DateTime::createFromFormat('d/m/Y H:i', $start_date_string);

        $formatterstart = new IntlDateFormatter(
            'it_IT',
            IntlDateFormatter::NONE,
            IntlDateFormatter::NONE,
            $start_date->getTimezone(),
            IntlDateFormatter::GREGORIAN,
            'MMMM'
        );

        $event_dates['start'] = [
            'hour' => $start_date->format('H'),
            'minute' => $start_date->format('i'),
            'day' => ltrim($start_date->format('d'), '0'),
            'month' => $formatterstart->format($start_date),
            'monthnum' => $start_date->format('m'),
            'year' => $start_date->format('Y')
        ];
    }

    // Verifica se la data di fine è stata fornita e non è vuota
    if (!empty($end_date_string)) {
        $end_date = DateTime::createFromFormat('d/m/Y H:i', $end_date_string);

        $formatterend = new IntlDateFormatter(
            'it_IT',
            IntlDateFormatter::NONE,
            IntlDateFormatter::NONE,
            $end_date->getTimezone(),
            IntlDateFormatter::GREGORIAN,
            'MMMM'
        );

        $event_dates['end'] = [
            'hour' => $end_date->format('H'),
            'minute' => $end_date->format('i'),
            'day' => ltrim($end_date->format('d'), '0'),
            'month' => $formatterend->format($end_date),
            'monthnum' => $end_date->format('m'),
            'year' => $end_date->format('Y')
        ];
    }

    // Se inserita solo la data di inizio, restituisci l'array delle date
    if (!empty($event_dates['start']) && (!isset($event_dates['end']) || empty($event_dates['end']))) {
        // Verifica se sia l'ora che i minuti sono entrambi 00
        if ($event_dates['start']['hour'] !== '00' || $event_dates['start']['minute'] !== '00') {
            // Stampa l'ora solo se entrambi non sono 00
            $event_dates['display'] =  
            // '<time class="grid gap-1" datetime="2024-10-12"><div class="evento-ora">'.$event_dates['start']['hour'].':'.$event_dates['start']['minute'].'</div><div class="block font-serif text-6xl">'.$event_dates['start']['day'].'</div><div class="italic font-bold text-2xl leading-none">'.$event_dates['start']['month'].'</div><div class="text-sm">'.$event_dates['start']['year'].'</div></time>';
            '<time datetime="'.$event_dates['start']['year'].'-'.$event_dates['start']['monthnum'].'-'.$event_dates['start']['day'].'">'.$event_dates['start']['day'].' '.$event_dates['start']['month'].' '.$event_dates['start']['year'].'</time>';
        } else {
            // Non stampare l'ora se entrambi sono 00
            $event_dates['display'] =  
            '<time datetime="'.$event_dates['start']['year'].'-'.$event_dates['start']['monthnum'].'-'.$event_dates['start']['day'].'">'.$event_dates['start']['day'].' '.$event_dates['start']['month'].' '.$event_dates['start']['year'].'</time>';
        }
        return $event_dates;
    }

    // Se inserite entrambe le date
    elseif(!empty($event_dates['start']) && !empty($event_dates['end'])){

        // Se le date sono uguali
        if($event_dates['start'] == $event_dates['end']){
            $event_dates['display'] =  
            '<time datetime="'.$event_dates['start']['year'].'-'.$event_dates['start']['monthnum'].'-'.$event_dates['start']['day'].'"><div>'.$event_dates['start']['day'].'</div><div>'.$event_dates['start']['month'].'</div><div>'.$event_dates['start']['year'].'</div></time>';
            return $event_dates;

        // Se le date sono diverse
        } else {

            // Se l'anno è uguale
            if ($event_dates['start']['year'] == $event_dates['end']['year']) {

                // Se anche il mese è uguale
                if ($event_dates['start']['month'] == $event_dates['end']['month']) {

                    // Se anche il giorno è uguale
                    if ($event_dates['start']['day'] == $event_dates['end']['day']) {

                        $event_dates['display'] = 
                        // '<time class="grid gap-1" datetime="2024-10-12"><div class="evento-ora">'.$event_dates['start']['hour'].':'.$event_dates['start']['minute'].'</div> / <div class="evento-ora">'.$event_dates['end']['hour'].':'.$event_dates['end']['minute'].'</div><div class="block font-serif text-6xl">'.$event_dates['start']['day'].'</div><div class="italic font-bold text-2xl leading-none">'.$event_dates['start']['month'].'</div><div class="text-sm">'.$event_dates['start']['year'].'</div></time>';
                        '<time datetime="'.$event_dates['start']['year'].'-'.$event_dates['start']['monthnum'].'-'.$event_dates['start']['day'].'">'.$event_dates['start']['day'].' '.$event_dates['start']['month'].' '.$event_dates['start']['year'].' '.$event_dates['start']['hour'].':'.$event_dates['start']['minute'].' - '.$event_dates['end']['hour'].':'.$event_dates['end']['minute'].'</time>';
                        return $event_dates;
                    
                    // Se anno e mese sono uguali ma giorno diverso
                    } else {

                        $event_dates['display'] =
                        '<time datetime="'.$event_dates['start']['year'].'-'.$event_dates['start']['monthnum'].'-'.$event_dates['start']['day'].'/'.$event_dates['end']['year'].'-'.$event_dates['end']['monthnum'].'-'.$event_dates['end']['day'].'">'.$event_dates['start']['day'].' - '.$event_dates['end']['day'].' '.$event_dates['start']['month'].' '.$event_dates['start']['year'].'</time>';
                        return $event_dates;
                    }

                // Se mese è diverso e l'anno è uguale
                } else{
                    
                    $event_dates['display'] =
                    '<time datetime="'.$event_dates['start']['year'].'-'.$event_dates['start']['monthnum'].'-'.$event_dates['start']['day'].'/'.$event_dates['end']['year'].'-'.$event_dates['end']['monthnum'].'-'.$event_dates['end']['day'].'">'.$event_dates['start']['day'].' '.$event_dates['start']['month'].' - '.$event_dates['end']['day'].' '.$event_dates['end']['month'].' '.$event_dates['start']['year'].'</time>';    
                    return $event_dates;
                }

            // Se è tutto diverso
            } else {
                $event_dates['display'] =
                '<time datetime="'.$event_dates['start']['year'].'-'.$event_dates['start']['monthnum'].'-'.$event_dates['start']['day'].'/'.$event_dates['end']['year'].'-'.$event_dates['end']['monthnum'].'-'.$event_dates['end']['day'].'">'.$event_dates['start']['day'].' '.$event_dates['start']['month'].' '.$event_dates['start']['year'].' - '.$event_dates['end']['day'].' '.$event_dates['end']['month'].' '.$event_dates['end']['year'].'</time>';  
                return $event_dates;
            }
        }
    }

    // Restituisci l'array con le date di inizio e fine, se disponibili
    return $event_dates;
}

function get_all_other_taxonomies($postId){
    $term_list = [];
    $taxonomies = get_post_taxonomies($postId);
    foreach ($taxonomies as $taxonomy) {
        $terms = get_the_terms($postId, $taxonomy);
        if ($taxonomy !== 'area-tematica' && $taxonomy !== 'categoria-evento'){
            if ($terms && !is_wp_error($terms)) {
                foreach ($terms as $term) {
                    $term_list[] = [
                        'taxonomy' => $taxonomy,
                        'category_name' => $term->name,
                        'category_slug' => $term->slug,
                    ];
                }
            }
        }
    }
    return $term_list;
}

function get_categoria_evento($postId){
    $term_list = [];
    $taxonomy = 'categoria-evento';
    $terms = get_the_terms($postId, $taxonomy);
    if ($terms && !is_wp_error($terms)) {
        foreach ($terms as $term) {
            if($term->slug == 'next-generation-fest'){
                $background_cat = 'next-gen-fest-bg.svg';
                $classes_cat = 'next-generation-fest';
                $classes_lab = '';
            } elseif($term->slug == 'giovanisi-in-tour'){
                $background_cat = 'giovanisi-in-tour-bg.svg';
                $classes_cat = 'n7-bg-giovanisi-tour giovanisi-in-tour';
                $classes_lab = '';
            } elseif($term->slug == 'task-force'){
                $background_cat = 'task-force-event.svg';
                $classes_cat = 'task-force';
                $classes_lab = 'n7-category--filled n7-bg-task-force';
            } elseif($term->slug == 'giovanisi-fest'){
                $background_cat = 'gs_fest.svg';
                $classes_cat = 'giovanisi-fest';
                $classes_lab = 'n7-bg-giovanisi-fest n7-text-giovanisi-fest';
            } elseif($term->slug == 'giovanisicuri'){
                $background_cat = 'gssicuri.svg';
                $classes_cat = 'giovanisicuri';
                $classes_lab = 'n7-text-giovanisicuri';
            } else {
                $background_cat = '';
                $classes_cat = '';
                $classes_lab = '';
            }
            $term_list[] = [
                'taxonomy' => $taxonomy,
                'category_name' => $term->name,
                'category_slug' => $term->slug,
                'background_cat' => $background_cat,
                'classes_cat' => $classes_cat,
                'classes_lab' => $classes_lab,
            ];
        }
    }
    return $term_list;
}  


function get_area_tematica($postId){
    $term_list = [];
    $taxonomy = 'area-tematica';
    $terms = get_the_terms($postId, $taxonomy);
    
    if ($terms && !is_wp_error($terms)) {
        foreach ($terms as $term) {
            $category_color_acf = get_field('scelta_colore_aree_tematiche', $term);
            if(isset($category_color_acf) && !empty($category_color_acf)){
                if ($category_color_acf == 'yellow'){
                    $category_color = 'n7-bg-studio';
                } else if ($category_color_acf == 'pink'){
                    $category_color = 'n7-bg-partecipo';
                } else if ($category_color_acf == 'green'){
                    $category_color = 'n7-bg-impresa';
                } else if ($category_color_acf == 'blue'){
                    $category_color = 'n7-bg-lavoro';
                } else if ($category_color_acf == 'red'){
                    $category_color = 'bg-white';
                }
            } else {
                $category_color = '';
            }
            $term_list[] = [
                'taxonomy' => $taxonomy,
                'category_name' => $term->name,
                'category_slug' => $term->slug,
                'category_color' => $category_color,
            ];
        }
    }
    return $term_list;
}    

function default_query_giovanisi($post_type = 'post', $posts_per_page = -1, $order = 'DESC', $meta_query = array(), $tax_query = array(), $pagination = '', $order_by = '', $exclude_ids = array(), $s = '') {
    // Parametri di default per la query
    $args = array(
        'post_type'      => $post_type,
        'posts_per_page' => $posts_per_page,
        'post_status'    => 'publish',
        'order'          => $order,
        'orderby'        => $order_by,
        'meta_query'     => $meta_query,
        'tax_query'      => $tax_query,
        'paged'          => $pagination,
        'post__not_in'   => $exclude_ids,  
        's'              => $s
    );

    // Esegui la query
    $query = new \WP_Query($args);

    // Restituisci i risultati della query
    return $query->posts;
}

function generatePaginationLinks($totalPages, $currentPage) {
    if ($totalPages > 1) {
        $first_page_url = ($currentPage > 1) ? add_query_arg('paged', 1, get_permalink()) : '';
        $first_page_button = '<li><a class="n7-pagination__link n7-pagination__link-button rounded" href="'.$first_page_url.'" aria-label="Pagina iniziale"><<</a></li>';
        $last_page_url = ($currentPage < $totalPages) ? add_query_arg('paged', $totalPages, get_permalink()) : '';
        $last_page_button = '<li><a class="n7-pagination__link n7-pagination__link-button rounded" href="'.$last_page_url.'" aria-label="Pagina finale">>></a></li>';
        if($currentPage==$totalPages){
            $last_page_button = '';
        }
        $listString = paginate_links(array(
            'base' => get_pagenum_link(1) . '%_%',
            'format' => 'page/%#%',
            'current' => $currentPage,
            'total' => $totalPages,
            'prev_text' => '<',
            'next_text' => '>',
            'type' => 'list',
        ));

        //Replace <span>
        $listString = str_replace('<span', '<a', $listString);
        $listString = str_replace('</span>', '</a>', $listString);
        //Pagina Precedente
        $listString = str_replace('class="prev page-numbers"', 'class="prev n7-pagination__link n7-pagination__link-button rounded" aria-label="Pagina precedente"', $listString);
        //Pagina Successiva
        $listString = str_replace('class="next page-numbers"', 'class="next n7-pagination__link n7-pagination__link-button rounded" aria-label="Pagina successiva"', $listString);
        //Pagina corrente
        $listString = str_replace('class="page-numbers current"', 'class="current n7-pagination__link n7-pagination__link-button n7-pagination__link-text rounded n7-background-primary n7-border-primary text-white hover:n7-content-01" aria-label="Pagina corrente, Pagina '.$currentPage.'" aria-current="true" href="#"', $listString);
        //Dots
        $listString = str_replace('class="page-numbers dots', 'class="dots n7-pagination__link n7-pagination__link-button n7-pagination__link-text rounded" aria-label="" href="#"', $listString);
        //Pagina Normale
        $listString = str_replace('class="page-numbers"', 'class="default n7-pagination__link n7-pagination__link-button n7-pagination__link-text rounded"', $listString);
        //Replace <ul>
        $listString = str_replace('page-numbers', 'flex gap-2', $listString);
        //Add first li "Iniziale"
        $listString = str_replace('<li><a class="prev', $first_page_button.'<li><a class="prev', $listString);
        //Add last li "Iniziale"
        $listString = str_replace('</ul>', $last_page_button.'</ul>', $listString);

        // Componi la stringa finale dei link di paginazione
        $pagination = $listString;
        return $pagination;
    }
    return '';
}

function check_stato_bando($post_id){

    $dateTimeZone = new \DateTimeZone('Europe/Rome');

    $status = get_field('stato', $post_id);
    $stato = isset($status) && !empty($status) ? $status : 'Stato non inserito';

    //Preview Conditions
    $stato = preview_for_status($stato);

    if (is_array($stato) && isset($stato['value'])) {
        // Inizializza le variabili per la data di scadenza, la data di attivazione e il testo della scadenza
        $scadenza = get_field('scadenza', $post_id);
        $scadenza_date = get_field('gs_scadenza_date', $post_id);

        $differenza = '';
        $scadenza_indicazione = '';
        $current_date = new \DateTime('now', $dateTimeZone);
        $scadenza_calculate = '';
        
        if (isset($scadenza) && !empty($scadenza)) {
            $scadenza_calculate = \DateTime::createFromFormat('d/m/Y H:i', $scadenza, $dateTimeZone);
        } elseif (isset($scadenza_date) && !empty($scadenza_date)) {
            $scadenza_calculate = \DateTime::createFromFormat('d/m/Y H:i', $scadenza_date.' 23:59', $dateTimeZone);
        }
        
        if ($scadenza_calculate && $current_date < $scadenza_calculate) {
            // Calcola la differenza tra le date
            $differenza = $current_date->diff($scadenza_calculate);
        
            if ($differenza->format('%a') >= 0 && $differenza->format('%a') < 4) {
                if ($differenza->format('%a') == 0) {
                    if ($current_date->format('d/m') == $scadenza_calculate->format('d/m')) {
                        $scadenza_indicazione = 'oggi';
                    } else {
                        $scadenza_indicazione = 'domani';
                    }
                } elseif ($differenza->format('%a') == 1) {
                    if ($current_date->modify('+1 day')->format('d/m') == $scadenza_calculate->format('d/m')) {
                        $scadenza_indicazione = 'domani';
                    } else {
                        $scadenza_indicazione = 'tra 2 Giorni';
                    }
                } elseif ($differenza->format('%a') == 2) {
                    if ($current_date->modify('+2 days')->format('d/m') == $scadenza_calculate->format('d/m')) {
                        $scadenza_indicazione = 'tra 2 Giorni';
                    } else {
                        $scadenza_indicazione = 'tra 3 Giorni';
                    }
                } elseif ($differenza->format('%a') == 3) {
                    if ($current_date->modify('+3 days')->format('d/m') == $scadenza_calculate->format('d/m')) {
                        $scadenza_indicazione = 'tra 3 Giorni';
                    } else {
                        $scadenza_indicazione = '';
                    }
                }
            }
        }

        $scadenza_text = get_field('gs_scadenza_text', $post_id);
        $data_di_attivazione = get_field('data_di_attivazione', $post_id);
        $data_di_attivazione_data = get_field('data_di_attivazione_data', $post_id);
        $data_di_attivazione_data_ora = get_field('data_di_attivazione_data_e_ora', $post_id);
        $annuncio_graduatoria = get_field('gs_annuncio_grad', $post_id);
        // Costruisci la risposta in base allo stato del bando
        switch ($stato['value']) {
            case 'attivo':
                if (!empty($scadenza)) {
                    return ['slug' => 'attivo', 'label' => 'Scade', 'text' => $scadenza, 'giorni_mancanti' => $scadenza_indicazione, 'scadenza_grad' => ''];
                } elseif (!empty($scadenza_date)) {
                    return ['slug' => 'attivo', 'label' => 'Scade', 'text' => $scadenza_date, 'giorni_mancanti' => $scadenza_indicazione, 'scadenza_grad' => ''];
                } elseif (!empty($scadenza_text)) {
                    return ['slug' => 'attivo', 'label' => 'Scade', 'text' => $scadenza_text, 'giorni_mancanti' => $scadenza_indicazione, 'scadenza_grad' => ''];
                } else {
                    return ['slug' => 'attivo', 'label' => 'Attivo', 'text' => '', 'giorni_mancanti' => ''];
                }
                break;
            case 'in_attivazione':
                if (!empty($data_di_attivazione_data_ora)) {
                    return ['slug' => 'in_attivazione', 'label' => 'In attivazione', 'text' => $data_di_attivazione_data_ora, 'giorni_mancanti' => '', 'scadenza_grad' => ''];
                } elseif (!empty($data_di_attivazione_data)) {
                    return ['slug' => 'in_attivazione', 'label' => 'In attivazione', 'text' => $data_di_attivazione_data, 'giorni_mancanti' => '', 'scadenza_grad' => ''];
                } elseif (!empty($data_di_attivazione)) {
                    return ['slug' => 'in_attivazione', 'label' => 'In attivazione', 'text' => $data_di_attivazione, 'giorni_mancanti' => '', 'scadenza_grad' => ''];
                } else {
                    return ['slug' => 'in_attivazione', 'label' => 'In attivazione', 'text' => '', 'giorni_mancanti' => '', 'scadenza_grad' => ''];
                }
                break;
            case 'in_graduatoria':
                if (!empty($annuncio_graduatoria) && isset($annuncio_graduatoria)) {
                    if (!empty($scadenza)) {
                        return ['slug' => 'in_graduatoria', 'label' => 'Graduatoria in uscita', 'text' => '', 'giorni_mancanti' => '', 'scadenza_grad' => $scadenza];
                    } elseif (!empty($scadenza_date)) {
                        return ['slug' => 'in_graduatoria', 'label' => 'Graduatoria in uscita', 'text' => '', 'giorni_mancanti' => '', 'scadenza_grad' => $scadenza_date];
                    } elseif (!empty($scadenza_text)) {
                        return ['slug' => 'in_graduatoria', 'label' => 'Graduatoria in uscita', 'text' => '', 'giorni_mancanti' => '', 'scadenza_grad' => $scadenza_text];
                    } else {
                        return ['slug' => 'in_graduatoria', 'label' => 'Graduatoria in uscita', 'text' => '', 'giorni_mancanti' => '', 'scadenza_grad' => ''];
                    }
                } else {
                    if (!empty($scadenza)) {
                        return ['slug' => 'in_graduatoria', 'label' => 'In Graduatoria', 'text' => '', 'giorni_mancanti' => '', 'scadenza_grad' => $scadenza];
                    } elseif (!empty($scadenza_date)) {
                        return ['slug' => 'in_graduatoria', 'label' => 'In Graduatoria', 'text' => '', 'giorni_mancanti' => '', 'scadenza_grad' => $scadenza_date];
                    } elseif (!empty($scadenza_text)) {
                        return ['slug' => 'in_graduatoria', 'label' => 'In Graduatoria', 'text' => '', 'giorni_mancanti' => '', 'scadenza_grad' => $scadenza_text];
                    } else {
                        return ['slug' => 'in_graduatoria', 'label' => 'In Graduatoria', 'text' => '', 'giorni_mancanti' => '', 'scadenza_grad' => ''];
                    }
                }
                break;
            case 'archivio':
                if (!empty($scadenza)) {
                    return ['slug' => 'archivio', 'label' => 'Scaduto', 'text' => $scadenza, 'giorni_mancanti' => '', 'scadenza_grad' => ''];
                } elseif (!empty($scadenza_date)) {
                    return ['slug' => 'archivio', 'label' => 'Scaduto', 'text' => $scadenza_date, 'giorni_mancanti' => '', 'scadenza_grad' => ''];
                } elseif (!empty($scadenza_text)) {
                    return ['slug' => 'archivio', 'label' => 'Scaduto', 'text' => $scadenza_text, 'giorni_mancanti' => '', 'scadenza_grad' => ''];
                } else {
                    return ['slug' => 'archivio', 'label' => 'Scaduto', 'text' => '', 'giorni_mancanti' => '', 'scadenza_grad' => ''];
                }
                break;
            default:
                return ['slug' => 'attivo', 'label' => $stato, 'text' => '', 'giorni_mancanti' => '', 'scadenza_grad' => ''];
        }
    } else {
        return ['slug' => 'archivio', 'label' => 'Non settato', 'text' => '', 'giorni_mancanti' => '', 'scadenza_grad' => ''];
    }
}

function generate_pagination($currentpage, $totalpage){
    // Implementa la logica della paginazione qui
    $pagination_html = '<nav class="n7-pagination pagination" role="navigation" aria-label="Paginazione" data-current-page="1" data-max-page="' . $totalpage . '">';
    $pagination_html .= '<ul class="flex gap-2">';
    
    // Aggiungi la gestione dei pulsanti "Prima" e "Precedente" se necessario
    if ($currentpage > 1) {
        $pagination_html .= '<li><a class="first-page-net7 n7-pagination__link n7-pagination__link-button rounded" href="#" aria-label="Pagina iniziale"><<</a></li>';
        $pagination_html .= '<li><a class="prev-page-net7 n7-pagination__link n7-pagination__link-button rounded" href="#" aria-label="Pagina precedente"><</a></li>';
    }
    
    // Aggiungi i numeri di pagina
    if ($totalpage > 6) {
        // Implementa la logica per mostrare solo una parte dei numeri di pagina quando ce ne sono molti
        if ($currentpage <= 3) {
            for ($i = 1; $i <= 4; $i++) {
                $pagination_html .= '<li><a class="page-number-net7 n7-pagination__link n7-pagination__link-button n7-pagination__link-text rounded';
                $pagination_html .= ($i == $currentpage) ? ' n7-background-primary n7-border-primary text-white hover:n7-content-01' : '';
                $pagination_html .= '" href="#" aria-label="Pagina ' . $i . '">' . $i . '</a></li>';
            }
            $pagination_html .= '<li><span class="ellipsis">...</span></li>';
            $pagination_html .= '<li><a class="page-number-net7 n7-pagination__link n7-pagination__link-button n7-pagination__link-text rounded" href="#" aria-label="Pagina ' . $totalpage . '">' . $totalpage . '</a></li>';
        } elseif ($currentpage >= $totalpage - 2) {
            $pagination_html .= '<li><a class="page-number-net7 n7-pagination__link n7-pagination__link-button n7-pagination__link-text rounded" href="#" aria-label="Pagina 1">1</a></li>';
            $pagination_html .= '<li><span class="ellipsis">...</span></li>';
            for ($i = $totalpage - 3; $i <= $totalpage; $i++) {
                $pagination_html .= '<li><a class="page-number-net7 n7-pagination__link n7-pagination__link-button n7-pagination__link-text rounded';
                $pagination_html .= ($i == $currentpage) ? ' n7-background-primary n7-border-primary text-white hover:n7-content-01' : '';
                $pagination_html .= '" href="#" aria-label="Pagina ' . $i . '">' . $i . '</a></li>';
            }
        } else {
            $pagination_html .= '<li><a class="page-number-net7 n7-pagination__link n7-pagination__link-button n7-pagination__link-text rounded" href="#" aria-label="Pagina 1">1</a></li>';
            $pagination_html .= '<li><span class="ellipsis">...</span></li>';
            for ($i = $currentpage - 1; $i <= $currentpage + 1; $i++) {
                $pagination_html .= '<li><a class="page-number-net7 n7-pagination__link n7-pagination__link-button n7-pagination__link-text rounded';
                $pagination_html .= ($i == $currentpage) ? ' n7-background-primary n7-border-primary text-white hover:n7-content-01' : '';
                $pagination_html .= '" href="#" aria-label="Pagina ' . $i . '">' . $i . '</a></li>';
            }
            $pagination_html .= '<li><span class="ellipsis">...</span></li>';
            $pagination_html .= '<li><a class="page-number-net7 n7-pagination__link n7-pagination__link-button n7-pagination__link-text rounded" href="#" aria-label="Pagina ' . $totalpage . '">' . $totalpage . '</a></li>';
        }
    } else {
        for ($i = 1; $i <= $totalpage; $i++) {
            $pagination_html .= '<li><a class="page-number-net7 n7-pagination__link n7-pagination__link-button n7-pagination__link-text rounded';
            $pagination_html .= ($i == $currentpage) ? ' n7-background-primary n7-border-primary text-white hover:n7-content-01' : '';
            $pagination_html .= '" href="#" aria-label="Pagina ' . $i . '">' . $i . '</a></li>';
        }
    }
    
    // Aggiungi la gestione dei pulsanti "Successivo" e "Ultimo" se necessario
    if ($currentpage < $totalpage) {
        $pagination_html .= '<li><a class="next-page-net7 n7-pagination__link n7-pagination__link-button rounded" href="#" aria-label="Pagina successiva">></a></li>';
        $pagination_html .= '<li><a class="last-page-net7 n7-pagination__link n7-pagination__link-button rounded" href="#" aria-label="Pagina finale">>></a></li>';
    }
    
    $pagination_html .= '</ul></nav>';
    return new HtmlString($pagination_html);
}

function getAlignment($allineamentox, $allineamentoy) {
    $output = '';
    // Controllo e gestione dell'allineamento orizzontale
    if (!empty($allineamentox) && isset($allineamentox)) {
        switch ($allineamentox) {
            case 'centro':
                $output .= 'is-h-centered ';
                break;
            case 'destra':
                $output .= 'is-r-aligned ';
                break;
            case 'sinistra':
                $output .= 'is-l-aligned ';
                break;
        }
    }
    // Controllo e gestione dell'allineamento verticale
    if (!empty($allineamentoy) && isset($allineamentoy)) {
        switch ($allineamentoy) {
            case 'centro':
                $output .= 'is-v-centered ';
                break;
            case 'alto':
                $output .= 'is-t-aligned ';
                break;
            case 'basso':
                $output .= 'is-b-aligned ';
                break;
        }
    }
    return trim($output);
}

function generate_img_with_size($image_id, $scale = '', $area_tematica_slug = '') {
    $image_md = !empty(wp_get_attachment_image_src($image_id, 'medium')[0]) && isset(wp_get_attachment_image_src($image_id, 'medium')[0]) ? wp_get_attachment_image_src($image_id, 'medium')[0] : false ;
    $image_lg = !empty(wp_get_attachment_image_src($image_id, 'large')[0]) && isset(wp_get_attachment_image_src($image_id, 'large')[0]) ? wp_get_attachment_image_src($image_id, 'large')[0] : false ;
    $image_full = !empty(wp_get_attachment_image_src($image_id, 'full')[0]) && isset(wp_get_attachment_image_src($image_id, 'full')[0]) ? wp_get_attachment_image_src($image_id, 'full')[0] : false ;    
    $html = '';
    $classToAdd = ($scale == 'yes') ? 'object-scale-down' : 'object-cover';
    if ($image_full) {
        if ($image_lg) {
            $html .= '<source srcset="'.$image_lg.'" media="(max-width: 567px)">';
        }
        if ($image_lg) {
            $html .= '<source srcset="'.$image_lg.'" media="(max-width: 767px)">';
            $html .= '<source srcset="'.$image_lg.'" media="(min-width: 768px) and (max-width: 1023px)">';
            $html .= '<source srcset="'.$image_lg.'">';
        }
        $html .= '<img src="'.$image_full.'" alt="" class="w-full h-full ' . $classToAdd . ' object-center">';
    } elseif (isset($area_tematica_slug) && !empty($area_tematica_slug)) {
        if($area_tematica_slug == 'lavoro'){
            $html .= '<img src="' . asset('images/placeholder_lavoro.jpg') . '" loading="lazy" alt="" class="w-full h-full ' . $classToAdd . ' object-center">';
        }
        if($area_tematica_slug == 'partecipo'){
            $html .= '<img src="' . asset('images/placeholder_partecipo.jpg') . '" loading="lazy" alt="" class="w-full h-full ' . $classToAdd . ' object-center">';
        }
        if($area_tematica_slug == 'studio-e-mi-formo'){
            $html .= '<img src="' . asset('images/placeholder_studio_e_mi_formo.jpg') . '" loading="lazy" alt="" class="w-full h-full ' . $classToAdd . ' object-center">';
        }
        if($area_tematica_slug == 'faccio-impresa'){
            $html .= '<img src="' . asset('images/placeholder_faccio_impresa.jpg') . '" loading="lazy" alt="" class="w-full h-full ' . $classToAdd . ' object-center">';
        }
    } else {
        $html .= '<source srcset="'.asset('images/card-placeholder.png').', '.asset('images/card-placeholder@2x.png').' 2x">';
        $html .= '<img src="' . asset('images/card-placeholder.png') . '" loading="lazy" alt="" class="w-full h-full ' . $classToAdd . ' object-center">';
    }
    return $html;
}

function formatPostsStoria($posts){
    $formattedPosts = [];
    foreach ($posts as $post) {
        setup_postdata($post);
        $postTitle = get_the_title($post);
        $postTitleMain = get_field('titolo', $post->ID);
        $titolo_storia = isset($postTitleMain) && !empty($postTitleMain) ? $postTitleMain : '' ;
        $thumbnail = get_the_post_thumbnail_url($post, 'full');
        if (!isset($thumbnail) || empty($thumbnail)) {
            $thumbnail = '';
        }
        $thumbnail_medium = get_the_post_thumbnail_url($post, 'medium');
        if (!isset($thumbnail_medium) || empty($thumbnail_medium)) {
            $thumbnail_medium = '';
        }
        $thumbnail_small = get_the_post_thumbnail_url($post, 'thumbnail');
        if (!isset($thumbnail_small) || empty($thumbnail_small)) {
            $thumbnail_small = '';
        }
        $areetematiche = get_area_tematica($post->ID);
        $videoOStoria = get_field('video_o_storia', $post->ID);
        $permalink = get_permalink($post);
        $icona = getIconaCategoriaStoria($videoOStoria);
        $visual = get_field('storie_possibili_scelta_tipologia_card');
        $visualcard = isset($visual) && $visual == 'tipo_1' ? 'tipo_1' : 'tipo_2';
        $classes = isset($areetematiche[0]['category_color']) && !empty($areetematiche[0]['category_color']) ? $areetematiche[0]['category_color'] : '';
        $formattedPosts[] = [
            'id'                 => $post->ID,
            'nome_post'          => $postTitle,
            'titolo_post'        => $titolo_storia,
            'immagine_anteprima' => $thumbnail,
            'immagine_medium'    => $thumbnail_medium,
            'immagine_small'     => $thumbnail_small,
            'icona_categoria'    => $icona,
            'permalink'          => $permalink,
            'areetematiche'      => $areetematiche,
            'video_storia'       => $videoOStoria,
            'colore'             => $classes,
            'tipologia_card'     => $visualcard,
        ];
    }
    wp_reset_postdata();
    return $formattedPosts;
}

function getIconaCategoriaStoria($videoOStoria) {
    if ($videoOStoria == 'video') {
        return '#linear--video-alt';
    } else {
        return '#linear--book-open';
    }
}

function add_class_iframe($html) {
    $html = preg_replace('/<iframe(.*?)>/', '<iframe$1 class="w-full aspect-video">', $html);
    $html = preg_replace('/\s+(width|height)="[^"]*"/i', '', $html);
    return $html;
}

function get_archive_for_storie($slug){

    $lavoro = get_field('lavoro_pagina_snodo', 'options-general');
    $studio = get_field('studio_e_mi_formo_pagina_snodo', 'options-general');
    $partecipo = get_field('partecipo_pagina_snodo', 'options-general');
    $faccio_impresa = get_field('faccio_impresa_pagina_snodo', 'options-general');

    $lavoro_url = isset($lavoro) && !empty($lavoro) ? $lavoro : '#';
    $studio_url = isset($studio) && !empty($studio) ? $studio : '#';
    $partecipo_url = isset($partecipo) && !empty($partecipo) ? $partecipo : '#';
    $faccio_impresa_url = isset($faccio_impresa) && !empty($faccio_impresa) ? $faccio_impresa : '#';

    if ($slug == 'studio-e-mi-formo'){
        $slug = $studio_url;
    } elseif ($slug == 'partecipo'){
        $slug = $partecipo_url;
    } elseif ($slug == 'faccio-impresa'){
        $slug = $faccio_impresa_url;
    } elseif ($slug == 'lavoro'){
        $slug = $lavoro_url;
    } else {
        $slug = '#';
    }
    return $slug;
}

function getSocialIconBySlug($name){
    $socials = [];
    $social_array = get_field('socials', 'options-general');
    if($name == 'instagram'){
        $socials['link'] = !empty($social_array['instagram']['url']) && isset($social_array['instagram']['url']) ? $social_array['instagram']['url'] : '';
        $socials['icon'] = 'solid--instagram';
        $socials['name'] = !empty($social_array['instagram']['title']) && isset($social_array['instagram']['title']) ? $social_array['instagram']['title'] : '';
    }
    if($name == 'tiktok'){
        $socials['link'] = !empty($social_array['tik_tok']['url']) && isset($social_array['tik_tok']['url']) ? $social_array['tik_tok']['url'] : '';
        $socials['icon'] = 'solid--tik-tok';
        $socials['name'] = !empty($social_array['tik_tok']['title']) && isset($social_array['tik_tok']['title']) ? $social_array['tik_tok']['title'] : '';
    }
    if($name == 'youtube'){
        $socials['link'] = !empty($social_array['youtube']['url']) && isset($social_array['youtube']['url']) ? $social_array['youtube']['url'] : '';
        $socials['icon'] = 'solid--youtube';
        $socials['name'] = !empty($social_array['youtube']['title']) && isset($social_array['youtube']['title']) ? $social_array['youtube']['title'] : '';
    }
    if($name == 'facebook'){
        $socials['link'] = !empty($social_array['facebook']['url']) && isset($social_array['facebook']['url']) ? $social_array['facebook']['url'] : '';
        $socials['icon'] = 'solid--facebook';
        $socials['name'] = !empty($social_array['facebook']['title']) && isset($social_array['facebook']['title']) ? $social_array['facebook']['title'] : '';
    }
    if($name == 'x'){
        $socials['link'] = !empty($social_array['x']['url']) && isset($social_array['x']['url']) ? $social_array['x']['url'] : '';
        $socials['icon'] = 'solid--x';
        $socials['name'] = !empty($social_array['x']['title']) && isset($social_array['x']['title']) ? $social_array['x']['title'] : '';
    }
    if($name == 'telegram'){
        $socials['link'] = !empty($social_array['telegram']['url']) && isset($social_array['telegram']['url']) ? $social_array['telegram']['url'] : '';
        $socials['icon'] = 'solid--telegram';
        $socials['name'] = !empty($social_array['telegram']['title']) && isset($social_array['telegram']['title']) ? $social_array['telegram']['title'] : '';
    }
    if($name == 'threads'){
        $socials['link'] = !empty($social_array['threads']['url']) && isset($social_array['threads']['url']) ? $social_array['threads']['url'] : '';
        $socials['icon'] = 'threads';
        $socials['name'] = !empty($social_array['threads']['title']) && isset($social_array['threads']['title']) ? $social_array['threads']['title'] : '';
    }
    if($name == 'linkedin'){
        $socials['link'] = !empty($social_array['linkedin']['url']) && isset($social_array['linkedin']['url']) ? $social_array['linkedin']['url'] : '';
        $socials['icon'] = 'solid--linkedin';
        $socials['name'] = !empty($social_array['linkedin']['title']) && isset($social_array['linkedin']['title']) ? $social_array['linkedin']['title'] : '';
    }
    return $socials;
}

function preview_for_status($stato) {
    if (is_preview()) {
        global $post;
        $preview_url = get_permalink($post->ID);
        $preview_url = add_query_arg('preview', 'true', $preview_url);
        $preview_nonce_url = wp_nonce_url($preview_url, 'post_preview_' . $post->ID);
        $preview_nonce_url_clear = str_replace("&amp;", "&", $preview_nonce_url);
        $preview_nonce_url_clear = str_replace("&_wpnonce", "&preview_nonce", $preview_nonce_url_clear);
        // Prendi il parametro nonce
        $query = parse_url($preview_nonce_url_clear, PHP_URL_QUERY);
        parse_str($query, $params);
        $nonce = $params['preview_nonce'];

        // Verifica il nonce
        if (isset($_GET['preview_nonce']) && !empty($_GET['preview_nonce']) && $_GET['preview_nonce'] == $nonce) {
            // Assegna il valore allo stato se esiste
            if (isset($_GET['preview_status']) && !empty($_GET['preview_status'])) {
                if ($_GET['preview_status'] == 'attivo') {
                    $stato = array('value' => 'attivo', 'label' => 'Attivo');
                } elseif ($_GET['preview_status'] == 'in_attivazione') {
                    $stato = array('value' => 'in_attivazione', 'label' => 'In attivazione');
                }
            }
        }
    }
    return $stato;
}

function print_preview_customize() {
    $html_preview = '';
    global $post;
    if (isset($_GET['is_main_preview']) && $_GET['is_main_preview'] == 'yes') {
        $stato = preview_for_status(null);
        if ($stato) {
            $device = '';
            if (isset($_GET['preview_device']) && !empty($_GET['preview_device'])) {
                if ($_GET['preview_device'] == 'mobile') {
                    $device = 'mobile';
                } elseif ($_GET['preview_device'] == 'desktop') {
                    $device = 'desktop';
                }
            }
            $is_main_preview = 'no';
            if (isset($_GET['is_main_preview']) && !empty($_GET['is_main_preview'])) {
                if ($_GET['is_main_preview'] == 'yes') {
                    $is_main_preview = 'yes';
                }
            }

            if ((($device == 'mobile') || ($device == 'desktop')) && (($stato['value'] == 'attivo') || ($stato['value'] == 'in_attivazione'))) {
                global $post;
                $preview_url = get_permalink($post->ID);
                $preview_url = add_query_arg('preview', 'true', $preview_url);
                $preview_nonce_url = wp_nonce_url($preview_url, 'post_preview_' . $post->ID);
                $preview_nonce_url_clear = str_replace("&amp;", "&", $preview_nonce_url);
                $preview_nonce_url_clear = str_replace("&_wpnonce", "&preview_nonce", $preview_nonce_url_clear);

                if ($device == 'mobile') {
                    $html_preview .= '
                    <div class="preview-container">
                        <div class="mobile-preview-frame">
                            <div class="mobile-preview">
                                <iframe class="preview-iframe" src="'.$preview_nonce_url_clear.'&preview_status='.$stato['value'].'&preview_device='.$device.'&is_main_preview=no" title="description"></iframe>
                            </div>
                            <span class="mobile-controls">
                                <img src="' . asset('images/phone-back.png') . '" class="control-icon">
                                <img src="' . asset('images/phone-home.png') . '" class="control-icon">
                                <img src="' . asset('images/phone-menu.png') . '" class="control-icon">
                            </span>
                        </div>';
                } elseif ($device == 'desktop') {
                    $html_preview .= '
                    <div class="preview-container">
                        <div class="desktop-preview-frame">
                            <div class="desktop-preview">
                                <iframe class="preview-iframe" src="'.$preview_nonce_url_clear.'&preview_status='.$stato['value'].'&preview_device='.$device.'&is_main_preview=no" title="description"></iframe>
                            </div>
                            <span class="desktop-logo" style="background-image: url(' . asset('images/footer/logo-footer.png') . ');"></span>
                        </div>';
                }
                if ($is_main_preview == 'yes') {
                    $html_preview .= '
                    <span class="preview-label">Stai visualizzando la anteprima '.$device.' per lo stato ' . $stato['label'] . ' (<a href="/wp/wp-admin/post.php?post='.$post->ID.'&action=edit" style="text-decoration:underline">chiudi</a>)</span>';
                }
                $html_preview .=  '
                    </div>
                    <style>
                    .preview-container {
                        display: flex;
                        justify-content: center;
                        align-items: center;
                        height: 100vh;
                        position: fixed;
                        background: white;
                        width: 100%;
                        top: 0;
                        left: 0;
                        z-index: 999999;
                    }
                    
                    .mobile-preview-frame,
                    .desktop-preview-frame {
                        position: relative;
                        border: 20px solid black;
                        border-radius: 30px;
                        border-inline-style: none;
                        border-left: 3px solid black;
                        border-right: 3px solid black;
                        display:flex;
                        justify-content:center;
                    }
                    .mobile-preview-frame{
                        overflow: hidden;
                    }

                    .desktop-preview-frame {
                        border-bottom-width: 45px;
                    }
                    
                    .mobile-preview {
                        background: white;
                        border-radius: 0;
                        overflow: hidden;
                        width: 375px;
                        height: 667px;
                    }
                    
                    .desktop-preview {
                        background: white;
                        border-radius: 0;
                        overflow: hidden;
                        width: 1350px;
                        height: 667px;
                    }
                    
                    .preview-iframe {
                        border: 0;
                        background: none;
                        overflow: scroll;
                        box-sizing: content-box;
                        width: calc(100% + 17px);
                        height: 100%;
                    }
                    
                    .mobile-controls {
                        width: 100%;
                        height: 40px;
                        bottom: 0;
                        background-color: #dddddd;
                        position: absolute;
                        z-index: 99;
                        display: flex;
                        justify-content: space-around;
                        align-items: center;
                    }
                    
                    .control-icon {
                        width: 26px;
                        height: 26px;
                    }
                    
                    .desktop-logo {
                        position: absolute;
                        width: 110px;
                        height: 35px;
                        bottom: -38px;
                        background-size: cover;
                    }
                    
                    .preview-label {
                        position: absolute;
                        top: 20px;
                        right: 0;
                        color: white;
                        background: #050406;
                        padding: 8px 12px;
                        font-family: Poppins;
                        border-radius: 10px 0 0 10px;
                        line-height: 1;
                        font-size: 14px;
                    }
                    body{
                        overflow: hidden;
                    }
                    </style>';
                echo $html_preview;
            }
        }
    } elseif (isset($_GET['is_main_preview']) && $_GET['is_main_preview'] == 'no') {
        echo '<style>
        html.scroll-smooth { --wpadminbar-height: 0px !important; margin-top: 0px !important; }
        body { padding: 0; margin: 0; }
        #wpadminbar { display:none; }
        </style>';
    }
}
