<?php

/**
* Map filters.
*/

namespace App;

add_action('mappress_map_display', function($map){
    $tax_query = array();
    $current_timestamp = time();

    $scelta_della_categoria_map = get_field('scegli_la_categoria_mappa'); //Blocco Map
    $scelta_della_categoria_map_advanced = get_field('seleziona_tipologia_di_evento_tab1_mappa'); //Blocco Map Advanced

    //Scelta della categoria
    if (isset($scelta_della_categoria_map) && !empty($scelta_della_categoria_map) && $scelta_della_categoria_map !== 'all') {
        $tax_query[] = array(
            'taxonomy' => 'categoria-evento',
            'field' => 'term_id',
            'terms' => $scelta_della_categoria_map,
        );
    } elseif (isset($scelta_della_categoria_map_advanced) && !empty($scelta_della_categoria_map_advanced) && $scelta_della_categoria_map_advanced != 'all'){
        $tax_query[] = array(
            'taxonomy' => 'categoria-evento',
            'field' => 'slug',
            'terms' => $scelta_della_categoria_map_advanced,
        );
    }

    $scelta_dell_area_tematica = get_field('scegli_area_tematica_mappa');
    if (!empty($scelta_dell_area_tematica) && $scelta_dell_area_tematica != 'all') {
        $tax_query[] = array(
            'taxonomy' => 'area-tematica',
            'field' => 'term_id',
            'terms' => $scelta_dell_area_tematica,
        );
    }

    if (empty($tax_query)) {
        $tax_query = array('relation' => 'AND');
    }

    $scelta_eventi_personalizzati = get_field('scegli_eventi_mappa');
    if (!empty($scelta_eventi_personalizzati)) {
        process_custom_events($map, $scelta_eventi_personalizzati);
    } else {
        process_default_events($map, $tax_query, $current_timestamp);
    }
});

function process_custom_events($map, $events) {
    foreach ($events as $event) {
        add_event_to_map($map, $event->ID);
    }
}

function process_default_events($map, $tax_query, $current_timestamp) {

    //MAP
    $cat_default_mappa = get_field('scegli_la_categoria_mappa');
    $area_default_mappa = get_field('scegli_area_tematica_mappa');

    //MAP ADVANCED
    $mostrafuturi_map = get_field('mostra_solo_futuri_tab1_mappa');
    $mostrasolofuturi_map = isset($mostrafuturi_map) && !empty($mostrafuturi_map) ? $mostrafuturi_map : false; // true/false
    $scelta_ordine_map = get_field('ordine_di_visualizzazione_tab1_mappa');
    $scelta_della_categoria_map_advanced = get_field('seleziona_tipologia_di_evento_tab1_mappa');
    $categoria_map_advanced = isset($scelta_della_categoria_map_advanced) && !empty($scelta_della_categoria_map_advanced) ? $scelta_della_categoria_map_advanced : '';

    // Imposta la meta query per mostrare tutti gli eventi o solo i futuri
    if (!$mostrasolofuturi_map) {
        // Mostra tutti gli eventi
        $meta_query = array(
            array('key' => 'luogo_evento', 'compare' => 'EXISTS'),
        );
    } else {
        // Mostra solo i futuri
        $meta_query = array(
            array('key' => 'luogo_evento', 'compare' => 'EXISTS'),
            array('key' => 'evento_data_inizio', 'value' => date('Y-m-d H:i:s', $current_timestamp), 'compare' => '>', 'type' => 'DATETIME'),
        );
    }

    // Imposta la tax query per la categoria
    $tax_query = [];
    if (!empty($categoria_map_advanced) && $categoria_map_advanced != 'all') {
        $tax_query[] = array(
            'taxonomy' => 'categoria-evento',
            'field' => 'slug',
            'terms' => $categoria_map_advanced,
        );
    } elseif ((!empty($cat_default_mappa) && isset($cat_default_mappa)) || (!empty($area_default_mappa) && isset($area_default_mappa))){
        if(!empty($cat_default_mappa) && isset($cat_default_mappa)){
            $tax_query[] = array(
                'taxonomy' => 'categoria-evento',
                'field' => 'term-id',
                'terms' => $cat_default_mappa,
            );
        }
        if(!empty($area_default_mappa) && isset($area_default_mappa)){
            $tax_query[] = array(
                'taxonomy' => 'area-tematica',
                'field' => 'term-id',
                'terms' => $area_default_mappa,
            );
        }
    }

    $eventi_map_advanced = default_query_giovanisi('evento', -1, 'DESC', $meta_query, $tax_query, '', '', array(), '');

    // Ordina i post per evento_data_inizio
    usort($eventi_map_advanced, function($a, $b) use ($scelta_ordine_map) {
        // Recupera i valori del campo meta evento_data_inizio
        $data_a = get_field('evento_data_inizio', $a->ID);
        $data_b = get_field('evento_data_inizio', $b->ID);

        // Converte le date nel formato Y-m-d H:i per facilitare il confronto
        $datetime_a = \DateTime::createFromFormat('d/m/Y H:i', $data_a);
        $datetime_b = \DateTime::createFromFormat('d/m/Y H:i', $data_b);

        if ($datetime_a == $datetime_b) {
            return 0;
        }

        if ($scelta_ordine_map == 'ASC') {
            return ($datetime_a < $datetime_b) ? -1 : 1;
        } else {
            return ($datetime_a > $datetime_b) ? -1 : 1;
        }
    });

    if(isset($scelta_ordine_map) && !empty($scelta_ordine_map) && $scelta_ordine_map == 'DESC'){
        array_reverse($eventi_map_advanced);
    }

    foreach($eventi_map_advanced as $eventi){
        add_event_to_map($map, $eventi->ID);
    }
}


function add_event_to_map($map, $event_id) {
    $place = get_field('luogo_evento', $event_id);
    if ($place) {
        $riassunto_anteprima = get_field('riassunto_anteprima', $event_id);
        $category_names = get_term_names($event_id, 'categoria-evento');
        $area_tematica_names = get_term_names($event_id, 'area-tematica');
        $data_evento = calculateDateEvent($event_id);

        $category_obj = get_the_terms($event_id, 'tipologia-evento');
        if ($category_obj && !is_wp_error($category_obj)) {
            $icon = get_field('icona_pin_mappa_tipologia', $category_obj[0]);
        } else {
            $icon = '';
        }
        $icon_pin = isset($icon) && !empty($icon) ? $icon.'.svg' : 'pin-red.svg';

        $visibilita = get_field('visibilita_orario_evento', $event_id);
        if(isset($visibilita) && $visibilita){
            $orarioevento = '<small class="mb-4 text-sm font-bold">Ore '. $data_evento['start']['hour'].':'.$data_evento['start']['minute'].'</small>';
        } else {
            $orarioevento = '';
        }

        $data_evento_complete = '';
        if(isset($data_evento['end']) && !empty($data_evento['end'])){
            if($data_evento['start']['year'] == $data_evento['end']['year']){
                if($data_evento['start']['month'] == $data_evento['end']['month']){
                    if($data_evento['start']['day'] == $data_evento['end']['day']){
                        $data_evento_complete = $data_evento['start']['day'].' '.$data_evento['start']['month'].' '.$data_evento['start']['year'];
                    } else{
                        $data_evento_complete = 'dal '.$data_evento['start']['day'].' al '.$data_evento['end']['day'].' '.$data_evento['start']['month'].' '.$data_evento['start']['year'];
                    }
                } else{
                    $data_evento_complete = 'dal '.$data_evento['start']['day'].' '.$data_evento['start']['month'].' al '.$data_evento['end']['day'].' '.$data_evento['end']['month'].' '.$data_evento['start']['year'];
                }
            } else {
                $data_evento_complete = 'dal '.$data_evento['start']['day'].' '.$data_evento['start']['month'].' '.$data_evento['start']['year'].' al '.$data_evento['end']['day'].' '.$data_evento['end']['month'].' '.$data_evento['end']['year'];
            }
        } else {
            $data_evento_complete = $data_evento['start']['day'].' '.$data_evento['start']['month'].' '.$data_evento['start']['year'];
        }

        if(!empty(get_the_terms($event_id, 'tipologia-evento')[0]->name)){
            $tipologia = get_the_terms($event_id, 'tipologia-evento')[0]->name;
        } else {
            $tipologia = '';
        }
        
        $map->pois[] = new \Mappress_Poi(array(
            "iconid" => $icon_pin,
            "title" => get_the_title($event_id),
            "body" => $riassunto_anteprima,
            "props" => array(
                "data" => $data_evento['start']['day'].' '.$data_evento['start']['month'].' '.$data_evento['start']['year'],
                "category" => $category_names,
                "area_tematica" => $area_tematica_names,
                "luogo" => get_field('evento_citta', $event_id),
                "link" => get_permalink($event_id),
                "orario" => $orarioevento,
                "data_complete" => $data_evento_complete,
                "tipologia" => $tipologia,
            ),
            "point" => array("lat" => $place['lat'], "lng" => $place['lng']),
        ));
    }
}


// Aggiungi select box per le categorie
add_filter('acf/load_field/name=scegli_la_categoria_mappa', function ($field) {
    return load_taxonomy_field($field, 'categoria-evento');
});

// Aggiungi select box per le aree tematiche
add_filter('acf/load_field/name=scegli_area_tematica_mappa', function ($field) {
    return load_taxonomy_field($field, 'area-tematica');
});

// Funzione per caricare le opzioni del campo di selezione della tassonomia
function load_taxonomy_field($field, $taxonomy) {
    $terms = get_terms(array(
        'taxonomy' => $taxonomy,
        'hide_empty' => false,
    ));
    if (!$terms || is_wp_error($terms)) {
        return $field;
    }
    foreach ($terms as $term) {
        $field['choices'][$term->term_id] = $term->name;
    }
    $field['multiple'] = 1;
    return $field;
}

// Funzione per ottenere i nomi delle categorie o delle aree tematiche di un post
function get_term_names($post_id, $taxonomy) {
    $area_tematica = get_area_tematica($post_id);
    $categoria_evento = get_categoria_evento($post_id);
    // dd($categoria_evento);
    $names = '';
    if (!empty($area_tematica) && isset($area_tematica)) {
        $names .= '<small class="n7-category n7-category--area-tematica n7-category--filled n7-category--sm ' . $area_tematica[0]['category_color'] . '">' . $area_tematica[0]['category_name'] . '</small>';
    }
    if (!empty($categoria_evento) && isset($categoria_evento)) {
        $names .= '<small class="n7-category n7-category--area-tematica n7-category--filled n7-category--sm">' . $categoria_evento[0]['category_name'] . '</small>';
    }
    return $names;
}
