<?php

/**
 * Theme setup.
 */

namespace App;

use function Roots\bundle;

/**
 * Register the theme assets.
 *
 * @return void
 */
add_action('wp_enqueue_scripts', function () {
    bundle('app')->enqueue();
}, 100);


/**
 * Register the theme assets with the block editor.
 *
 * @return void
 */

// Editor scripts
add_action('enqueue_block_editor_assets', function () {
    global $post;
    $handle = 'editor';  // Handle per il tuo script dell'editor
    bundle($handle)->enqueue();  // Enqueue del bundle dell'editor

    wp_enqueue_script($handle, asset('scripts/editor.js')->uri(), ['jquery'], null, true);

    // Localizza il tuo script per passare parametri
    global $post;

    if (isset($post) && $post->post_type === 'bando') {
        // Prepara l'URL dell'anteprima
        $preview_url = get_permalink($post->ID);
        $preview_url = add_query_arg('preview', 'true', $preview_url);
        $preview_nonce_url = wp_nonce_url($preview_url, 'post_preview_' . $post->ID, 'preview_nonce');
        wp_localize_script($handle, 'editorSettings', [
            'preview_nonce_url' => add_query_arg('preview_id', $post->ID, $preview_nonce_url)
        ]);
    } else {
        wp_localize_script($handle, 'editorSettings', [
            
        ]);
    }

}, 100);

/**
 * Register the initial theme setup.
 *
 * @return void
 */
add_action('after_setup_theme', function () {
    /**
     * Disable full-site editing support.
     *
     * @link https://wptavern.com/gutenberg-10-5-embeds-pdfs-adds-verse-block-color-options-and-introduces-new-patterns
     */
    //remove_theme_support('block-templates');

    /**
     * Register the navigation menus.
     *
     * @link https://developer.wordpress.org/reference/functions/register_nav_menus/
     */
    register_nav_menus([
        'top_header' => __('Header Top', 'giovanisi'),
        'footer' => __('Footer', 'giovanisi'),
    ]);
    /**
     * Disable the default block patterns.
     *
     * @link https://developer.wordpress.org/block-editor/developers/themes/theme-support/#disabling-the-default-block-patterns
     */
    remove_theme_support('core-block-patterns');

    /**
     * Enable plugins to manage the document title.
     *
     * @link https://developer.wordpress.org/reference/functions/add_theme_support/#title-tag
     */
    add_theme_support('title-tag');

    /**
     * Enable post thumbnail support.
     *
     * @link https://developer.wordpress.org/themes/functionality/featured-images-post-thumbnails/
     */
    add_theme_support('post-thumbnails');

    /**
     * Enable responsive embed support.
     *
     * @link https://developer.wordpress.org/block-editor/how-to-guides/themes/theme-support/#responsive-embedded-content
     */
    add_theme_support('responsive-embeds');

    /**
     * Enable HTML5 markup support.
     *
     * @link https://developer.wordpress.org/reference/functions/add_theme_support/#html5
     */
    add_theme_support('html5', [
        'caption',
        'comment-form',
        'comment-list',
        'gallery',
        'search-form',
        'script',
        'style',
    ]);

    /**
     * Enable selective refresh for widgets in customizer.
     *
     * @link https://developer.wordpress.org/reference/functions/add_theme_support/#customize-selective-refresh-widgets
     */
    add_theme_support('customize-selective-refresh-widgets');
}, 20);

/**
 * Register the theme sidebars.
 *
 * @return void
 */
add_action('widgets_init', function () {
    $config = [
        'before_widget' => '<section class="widget %1$s %2$s">',
        'after_widget' => '</section>',
        'before_title' => '<h3>',
        'after_title' => '</h3>',
    ];

    register_sidebar([
        'name' => __('Primary', 'sage'),
        'id' => 'sidebar-primary',
    ] + $config);

    register_sidebar([
        'name' => __('Footer', 'sage'),
        'id' => 'sidebar-footer',
    ] + $config);
});

add_action('wp_footer', function () { ?>
    <script type="text/javascript">
        var ajax_object = {
            ajax_url: "<?php echo admin_url('admin-ajax.php'); ?>",
            nonce: "<?php echo wp_create_nonce('my_nonce'); ?>"
        };
    </script>
<?php });

// add_filter( 'wpseo_opengraph_image', function ( $image ) {
//     // Sostituisci con l'URL della tua immagine predefinita
//     $default_image = asset('images/placeholder.jpg');
//     // $default_image = asset('images/placeholder.png');
//     return $default_image;
// });