<?php
function getPlaylistVideos($maxResults = 10) {
    $apiKey = getenv('YOUTUBE_KEY');
    $playlistId = get_field('playlist_id_youtube_feed', 'options-general');

    if (!$playlistId || !$apiKey) {
        error_log('API key o Playlist ID mancante.');
        return [];
    }

    $apiUrl = "https://www.googleapis.com/youtube/v3/playlistItems?part=snippet&maxResults={$maxResults}&playlistId={$playlistId}&key={$apiKey}";
    $response = wp_remote_get($apiUrl);

    if (is_wp_error($response)) {
        error_log('Errore nel recupero dei dati dall\'API di YouTube: ' . $response->get_error_message());
        return [];
    }

    $body = wp_remote_retrieve_body($response);
    $videos = json_decode($body, true);

    if (isset($videos['error'])) {
        error_log('Errore API: ' . json_encode($videos['error']));
        return [];
    }

    if (!empty($videos['items'])) {
        update_option('my_playlist_videos', $videos['items'], false);
    } else {
        error_log('Nessun video trovato da salvare.');
    }
}

//Verifica che il bottone nella pagina opzioni sia stato premuto
add_action('admin_init', function() {
     if (isset($_GET['refresh']) && $_GET['refresh'] == 'yes') {
        $response = getPlaylistVideos();
        if (!empty($response)) {
            add_action('admin_notices', function() use ($response) {
                echo "<div class=\"notice notice-success is-dismissible\"><p>{$response['message']}</p></div>";
            });
        }
     }
});

//Aggiungi un form che wrappa il campo ACF Button per mandare parametro in GET 
function form_refresh_feed_button_acf() { ?>
    <script type="text/javascript">
        document.addEventListener('DOMContentLoaded', function() {
            var container = document.querySelector('.acf-field[data-name="refresh_feed_youtube_feed"]');

            if (container) {
                var form = document.createElement('form');
                form.action = window.location.pathname + window.location.search;
                form.method = 'GET';
                form.className = 'refresh-youtube';

                var params = new URLSearchParams(window.location.search);
                params.forEach(function(value, key) {
                    var input = document.createElement('input');
                    input.type = 'hidden';
                    input.name = key;
                    input.value = value;
                    form.appendChild(input);
                });

                var refreshInput = document.createElement('input');
                refreshInput.type = 'hidden';
                refreshInput.name = 'refresh';
                refreshInput.value = 'yes';
                form.appendChild(refreshInput);

                var nonceField = document.createElement('input');
                nonceField.type = 'hidden';
                nonceField.name = '_wpnonce';
                nonceField.value = '<?php echo wp_create_nonce('refresh_feed_nonce'); ?>';
                form.appendChild(nonceField);

                container.parentNode.insertBefore(form, container);
                form.appendChild(container);
            }
        });
    </script>
<?php }
add_action('acf/input/admin_footer', 'form_refresh_feed_button_acf');

// Registrazione dell'hook per il cron job
add_action('init', function() {
    // Non programmiamo l'evento se è già programmato
    if (!wp_next_scheduled('update_youtube_playlist_cron')) {
        wp_schedule_event(time(), 'six_hours', 'update_youtube_playlist_cron');
    }
});

// Aggiungere l'azione al cron job
add_action('update_youtube_playlist_cron', 'getPlaylistVideos');

// Aggiungere un intervallo personalizzato di sei ore
add_filter('cron_schedules', function($schedules) {
    $schedules['six_hours'] = array(
        'interval' => 6 * HOUR_IN_SECONDS,
        'display' => __('Every Six Hours')
    );
    return $schedules;
});