<?php

/*
|--------------------------------------------------------------------------
| Register The Auto Loader
|--------------------------------------------------------------------------
|
| Composer provides a convenient, automatically generated class loader for
| our theme. We will simply require it into the script here so that we
| don't have to worry about manually loading any of our classes later on.
|
*/

if (! file_exists($composer = __DIR__.'/vendor/autoload.php')) {
    wp_die(__('Error locating autoloader. Please run <code>composer install</code>.', 'sage'));
}

require $composer;

/*
|--------------------------------------------------------------------------
| Register The Bootloader
|--------------------------------------------------------------------------
|
| The first thing we will do is schedule a new Acorn application container
| to boot when WordPress is finished loading the theme. The application
| serves as the "glue" for all the components of Laravel and is
| the IoC container for the system binding all of the various parts.
|
*/

if (! function_exists('\Roots\bootloader')) {
    wp_die(
        __('You need to install Acorn to use this theme.', 'sage'),
        '',
        [
            'link_url' => 'https://roots.io/acorn/docs/installation/',
            'link_text' => __('Acorn Docs: Installation', 'sage'),
        ]
    );
}

\Roots\bootloader()->boot();

/*
|--------------------------------------------------------------------------
| Register Sage Theme Files
|--------------------------------------------------------------------------
|
| Out of the box, Sage ships with categorically named theme files
| containing common functionality and setup to be bootstrapped with your
| theme. Simply add (or remove) files from the array below to change what
| is registered alongside Sage.
|
*/

collect(['setup', 'filters', 'customizer', 'tinymce-customize', 'map', 'customize-admin', 'helpers', 'ajax', 'menu-utilities', 'youtube-feed', 'mail-template', 'wpcron'])
    ->each(function ($file) {
        // Includi tutti i file tranne acf.php quando non siamo in produzione
        if ($file !== 'acf' || (defined('WP_ENV') && WP_ENV === 'production')) {
            // Verifica se il file esiste prima di includerlo
            if (!locate_template("app/{$file}.php", true, true)) {
                // Se il file non viene trovato, mostra un messaggio di avviso
                echo sprintf(__('Warning: unable to locate <code>%s</code> for inclusion.', 'sage'), "app/{$file}.php");
            }
        }
    });

// Include acf.php solo se siamo in produzione
if (defined('WP_ENV') && WP_ENV === 'production') {
    if (!locate_template('app/acf.php', true, true)) {
        // Se il file non viene trovato, mostra un messaggio di avviso
        echo __('Warning: unable to locate <code>acf.php</code> for inclusion.', 'sage');
    }
}

function add_headers() {
    $csp = "default-src 'self' https://www.youtube.com https://www.youtube-nocookie.com; ";
    $csp .= "script-src 'self' 'unsafe-inline' 'unsafe-eval' https://ingestion.webanalytics.italia.it https://maps.googleapis.com https://unpkg.com https://cs.iubenda.com https://cdn.iubenda.com https://connect.facebook.net https://www.facebook.com https://analytics.tiktok.com https://www.youtube.com https://s.ytimg.com https://www.gstatic.com; ";
    $csp .= "connect-src 'self' https://maps.googleapis.com https://ingestion.webanalytics.italia.it https://hits-i.iubenda.com https://consent.iubenda.com https://idb.iubenda.com https://cpl.iubenda.com https://graph.facebook.com https://analytics.tiktok.com https://www.youtube.com https://youtubei.googleapis.com https://play.google.com; ";
    $csp .= "img-src 'self' data: https://ingestion.webanalytics.italia.it https://i.ytimg.com https://maps.gstatic.com https://maps.googleapis.com https://secure.gravatar.com https://www.facebook.com; ";
    $csp .= "style-src 'self' 'unsafe-inline' https://fonts.googleapis.com https://www.youtube.com; ";
    $csp .= "font-src 'self' data: https://fonts.gstatic.com https://ka-p.fontawesome.com; ";
    $csp .= "worker-src 'self' blob:; ";
    $csp .= "frame-src 'self' https://www.google.com https://www.youtube.com https://www.youtube-nocookie.com https://cdn.iubenda.com; ";
    $csp .= "media-src 'self' https://www.youtube.com; ";
    $csp .= "object-src 'self'; ";
    $csp .= "form-action 'self'; ";
    $csp .= "frame-ancestors 'self' https://giovanisi.it; ";
    $csp .= "report-uri /csp-report-endpoint.php;";

    // Imposta le intestazioni personalizzate
    header('X-Content-Type-Options: nosniff');
    header('X-Frame-Options: SAMEORIGIN');
    header('Strict-Transport-Security: max-age=31536000; includeSubDomains');
    header("Content-Security-Policy: $csp");
}
add_action('send_headers', 'add_headers');

function add_login_csp() {
    $csp = "default-src 'self'; ";
    $csp .= "script-src 'self' 'unsafe-inline'; ";
    $csp .= "style-src 'self' 'unsafe-inline'; ";
    $csp .= "img-src 'self' data:; ";
    $csp .= "font-src 'self' data:; ";
    $csp .= "connect-src 'self'; ";
    $csp .= "form-action 'self'; "; // Protegge contro il formjacking
    $csp .= "frame-ancestors 'none'; "; // Protegge contro il clickjacking
    $csp .= "object-src 'none'; ";
    $csp .= "report-uri /csp-report-endpoint.php;";

    header("Content-Security-Policy: $csp");
}
add_action('login_enqueue_scripts', 'add_login_csp');


add_filter('rest_endpoints', function($endpoints) {
    // Verifica se l'utente non è autenticato
    if (!is_user_logged_in()) {
        // Nascondi l'endpoint per la lista degli utenti
        if (isset($endpoints['/wp/v2/users'])) {
            unset($endpoints['/wp/v2/users']);
        }
        // Nascondi l'endpoint per un singolo utente
        if (isset($endpoints['/wp/v2/users/(?P<id>[\d]+)'])) {
            unset($endpoints['/wp/v2/users/(?P<id>[\d]+)']);
        }
    }
    return $endpoints;
});
