/* MOBILE DIALOG/MODAL - mobile management */
document.addEventListener('DOMContentLoaded', function () {

    var modals = [
        document.getElementById('mobileModalFilters'),
        document.getElementById('mapMobileModal')
    ];

    function modalMobileResize(modal) {
        if (modal) {
            if (window.matchMedia("screen and (max-width: 767px)").matches) {
                modal.setAttribute('role', 'dialog');
                // modal.setAttribute('aria-labelledby', 'title' + modal);
                modal.setAttribute('aria-modal', 'true');
            } else if (window.matchMedia("screen and (min-width: 768px)").matches) {
                modal.removeAttribute('role');
                modal.removeAttribute('aria-labelledby');
                modal.removeAttribute('aria-modal');
            }
        }
    }

    function applyResizeToModals() {
        modals.forEach(function (modal) {
            modalMobileResize(modal);
        });

    }

    window.addEventListener('resize', applyResizeToModals);
    applyResizeToModals();

    function initializeModal({ modalId, openButtonId, closeButtonId, showContentId }) {
        const modal = document.getElementById(modalId);
        const openModalBtn = document.getElementById(openButtonId);
        const closeModalBtn = document.getElementById(closeButtonId);
        const showContentBtn = showContentId ? document.getElementById(showContentId) : null;

        const body = document.body;

        if (openModalBtn) {
            openModalBtn.addEventListener('click', function () {
                modal.classList.remove('hidden');
                modal.classList.add('z-[2000]');
                body.classList.add('has-modal-open');
                const focusableModalElements = modal.querySelectorAll('button, [href], input, select, textarea, [tabindex]:not([tabindex="-1"])');
                if (focusableModalElements.length > 0) {
                    focusableModalElements[0].focus();
                }
                document.addEventListener('keydown', trapTabKey);
            });
        }


        const closeFunction = function () {
            modal.classList.add('hidden');
            modal.classList.remove('z-[2000]');
            body.classList.remove('has-modal-open');
            openModalBtn.focus();
            document.removeEventListener('keydown', trapTabKey);
        };

        if (closeModalBtn) {
            closeModalBtn.addEventListener('click', closeFunction);
        }

        if (showContentBtn) {
            showContentBtn.addEventListener('click', closeFunction);
        }

        function trapTabKey(e) {
            if (!modal.contains(document.activeElement)) {
                return;
            }

            const focusableModalElements = modal.querySelectorAll('button, [href], input, select, textarea, [tabindex]:not([tabindex="-1"])');
            const firstElement = focusableModalElements[0];
            const lastElement = focusableModalElements[focusableModalElements.length - 1];

            if (e.key === 'Tab') {
                if (e.shiftKey) /* shift + tab */ {
                    if (document.activeElement === firstElement) {
                        e.preventDefault();
                        lastElement.focus();
                    }
                } else /* tab */ {
                    if (document.activeElement === lastElement) {
                        e.preventDefault();
                        firstElement.focus();
                    }
                }
            }
        }

        document.addEventListener('keydown', function (event) {
            if (modal) {
                if (event.key === 'Escape' && !modal.classList.contains('hidden')) {
                    modal.classList.add('hidden');
                    body.classList.remove('has-modal-open');
                    openModalBtn.focus();
                    document.removeEventListener('keydown', trapTabKey);
                }
            }
        });
    }


    // Utilizzo della funzione per una modale specifica
    initializeModal({
        modalId: 'mobileModalFilters',
        openButtonId: 'mobileModalTrigger',
        closeButtonId: 'mobileModalClose',
        showContentId: 'mobileModalShow'
    });

    initializeModal({
        modalId: 'mapMobileModal',
        openButtonId: 'openMapModalBtn',
        closeButtonId: 'closeMapModalBtn'
    });

});