/** @type {import('tailwindcss').Config} config */
const config = {
  content: ['./app/**/*.php', './resources/**/*.{php,vue,js}'],
  safelist: [
    'md:w-1/2',
    'md:w-1/3',
    'md:w-1/4',
    'mapp-layout',
    'mapp-wrapper',
    'mapp-list-header',
    'mapp-sidebar',
    'mapp-template',
    'mapp-list',
    'mapp-info',
    'mapp-item',
    'mapp-ib',
    'mapp-ib-content',
    'mapp-menu-toggle',
    'mapp-info',
    'mapp-iw',
    'mapp-footer',
    'mapp-button',
    'mapp-view-list',
    'mapp-selected',
    'mapp-view-map',
    'n7-form-item--privacy',
    'wp-block-separator',
    'iubenda-tp-btn'
  ],
  theme: {
    aspectRatio: {
      auto: 'auto',
      square: '1 / 1',
      video: '16 / 9',
      1: '1',
      2: '2',
      3: '3',
      4: '4',
      5: '5',
      6: '6',
      7: '7',
      8: '8',
      9: '9',
      10: '10',
      11: '11',
      12: '12',
      13: '13',
      14: '14',
      15: '15',
      16: '16',
    },
    fontFamily: {
      'sans': ['Poppins', 'sans-serif'],
      'serif': ['Arsenica', 'serif'],
    },
    colors: {
      transparent: 'transparent',
      current: 'currentColor',
      black: '#000000',
      white: '#ffffff',
      'shiny-yellow':'#FFD600',
      'deep-blue':'#8396FD',
      'acid-green':'#6DEC4C',
      'fucsia':'#FF66D4',
      'light-orange':'#FBB03B',
      'shiny-orange':'#FF7144',
      'dark-purple': '#3D3C74',
      'mauve': '#B65CCF',
      'light-green': '#40C88C',
      'neutral-orange':'#F0782C',
      'pale-yellow': '#FFDB94',
      'intense-light-green':'#5EF2BD',
      'teal':'#00A499',
      'darker-teal':'#006D67',
      'bright-red':'#e02c2f',
      success: {
        dark: '#166534',
        DEFAULT: '#22c55e',
        light: '#bbf7d0',
      },
      error: {
        dark: '#991b1b',
        DEFAULT: '#ef4444',
        light: '#fecaca',
      },
      warn: {
        dark: '#9a3412',
        DEFAULT: '#f97316',
        light: '#fed7aa',
      },
      info: {
        dark: '#075985',
        DEFAULT: '#0ea5e9',
        light: '#bae6fd',
      },
      primary: {
        50: '#FBCFCF',
        100: '#F8AFAF',
        200: '#F58787 ',
        300: '#F15E5E',
        400: '#EE3636',
        500: '#EA0E0E',
        600: '#C30C0C',
        700: '#9C0909',
        800: '#750707',
        900: '#4E0505'
      },
      secondary: {
        50: '#E6E0FA',
        100: '#D5CCF7',
        200: '#BFB2F3',
        300: '#AA98EF',
        400: '#957FEB',
        500: '#8065E7',
        600: '#6B54C0',
        700: '#55439A',
        800: '#403374',
        900: '#2B224D'
      },
      neutral: {
        50: '#FCFBFF',
        100: '#FAF9FF',
        200: '#F7F5FF',
        300: '#F4F2FF',
        400: '#F2EFFF',
        500: '#EFECFF',
        600: '#C7C5D4',
        700: '#9F9DAA',
        800: '#787680',
        900: '#504F55'
      },
      brand: {
        DEFAULT: '#E50000'
      },
      focusring: {
        DEFAULT: '#025ECC'
      }
    },
    extend: {
      fontSize: {
        '6xl': ['3.25rem',{
          lineHeight: '1.2'
        }],
        '4xl': ['2rem',{
          lineHeight: '1.2'
        }]
      },
      spacing: {
        '1e': '1em',
      },
      lineHeight: {
        'medium': '1.57'
      },
      boxShadow: {
        'red-outline': '2px 2px 0 0px #E50000',
        'padded-text': '0.25rem 0 0 black, -0.25rem 0 0 black'
      },
      backgroundImage: {
        'gradient-01': 'linear-gradient(90deg, #024FE5 18.19%, #B12FD1 48%, #CE2121 79.02%)',
        'gradient-02': 'linear-gradient(180deg, #FF007A 0%, rgba(133, 102, 255, 0.60) 100%)',
        'gradient-03': 'linear-gradient(330deg, #FF4F51 13.19%, rgba(195, 255, 98, 0.10) 100.02%)',
        'gradient-04': 'linear-gradient(262deg, rgba(27, 11, 79, 0.63) 23.04%, rgba(255, 22, 232, 0.00) 86.54%), linear-gradient(115deg, rgba(52, 47, 73, 0.80) 33.86%, rgba(72, 49, 103, 0.80) 62.36%, rgba(255, 33, 126, 0.24) 95.24%)',
        'gradient-05': 'linear-gradient(356deg, rgba(0, 0, 0, 0.50) 6.96%, rgba(0, 0, 0, 0.00) 93.15%)',
        'gradient-06': 'linear-gradient(140deg, rgba(199, 2, 2, 0.24) 23.02%, rgba(46, 2, 173, 0.24) 86.13%)',
        'gradient-07':'linear-gradient(110deg, #401489 15.94%, #543097 49.08%, #2C1557 85.74%)',
        'gradient-speaker': 'linear-gradient(128deg, #45cb91 25%, #bb22e5 80%)',
      },
      skew: {
        '15': '15deg',
      },
      flex: {
        '2': '2 2 0%',
        '3': '3 3 0%',
      },
      filter: {
        'padded-effect': "url('#paddedEffect')",
      },
      screens: {
        'xs': {'raw': '(min-width: 425px)'},
        'md-max-w-480-min-h': {'raw': '(max-width: 1023px) and (min-height: 480px)'},
        'lg-min-w-480-min-h': {'raw': '(min-width: 1024px) and (min-height: 480px)'},
        'xl-min-w-480-min-h': {'raw': '(min-width: 1280px) and (min-height: 480px)'},
        'xl-max-w-480-min-h': {'raw': '(max-width: 1279px) and (min-height: 480px)'},
        'min-h-480': {'raw': '(min-height: 480px)'},
        'min-w-1536': {'raw': '(min-width: 1536px)'},
        'min-w-1700': {'raw': '(min-width: 1700px)'},
      },
      gridTemplateColumns: {
        '1-2': '1fr 2fr',
        '2-1': '2fr 1fr',
        '1-3': '1fr 3fr'
      },
      keyframes: {
        'blur-on': {
          '0%': { opacity: '0' },
          '100%': { opacity: '1' },
        }
      },
      animation: {
        'blur-on': 'blur-on 0.33s ease-in-out',
      }
    },
    container: {
      center: true,
      padding: {
        DEFAULT: '1rem',
      },
    },
  },
  variants: {
    translate: ['responsive', 'hover', 'focus', 'active', 'group-hover'],

    extend: {
      backgroundColor: ['active'],
      borderColor: ['active']
    }
  },
  plugins: [
    require('@tailwindcss/aspect-ratio'),
    require('@tailwindcss/forms'),
    require('@tailwindcss/typography'),
  ],
};

export default config;
